# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

import contextlib

from networking_vsphere.common import constants
from networking_vsphere.common import error
from networking_vsphere.common import model
from networking_vsphere.drivers import dvs_driver
from networking_vsphere.tests import base
from networking_vsphere.tests.unit.utils import fake_vmware_api
from networking_vsphere.tests.unit.utils import stubs
from networking_vsphere.utils import error_util
from networking_vsphere.utils import network_util
from networking_vsphere.utils import resource_util
from networking_vsphere.utils import vim_util


class TestDvsDriver(base.TestCase):

    def setUp(self):
        super(TestDvsDriver, self).setUp()
        self.cluster_dvs_mapping = {"ClusterComputeResource": "test_dvs"}
        self.fake_visdk = self.useFixture(stubs.FakeVmware())
        self.session = self.fake_visdk.session
        self.useFixture(stubs.CacheFixture())
        self.vc_driver = dvs_driver.DvsNetworkDriver()
        self.vc_driver.state = constants.DRIVER_RUNNING
        self.vc_driver.add_cluster("ClusterComputeResource", "test_dvs")

    def test_invalid_cluster(self):
        valid, _ = self.vc_driver.validate_cluster_switch_mapping(
            "InvalidClusterComputeResource", "test_dvs")
        self.assertFalse(valid)

    def test_invalid_dvs(self):
        valid, _ = self.vc_driver.validate_cluster_switch_mapping(
            "ClusterComputeResource", "invalid_dvs")
        self.assertFalse(valid)

    def test_create_network(self):
        vlan = model.Vlan(vlanIds=["1001"])
        network_config = model.NetworkConfig(vlan)
        network = model.Network(
            name="net-1234", network_type=constants.NETWORK_VLAN,
            config=network_config)
        vswitch = model.VirtualSwitch("test_dvs", hosts=None)
        with mock.patch.object(network_util, "get_portgroup_mor_by_name",
                               return_value=None):
            self.vc_driver.create_network(network, vswitch)
            self.assertTrue(fake_vmware_api.is_task_done(
                "AddDVPortgroup_Task"))

    def test_delete_network(self):
        pg_name = fake_vmware_api.Constants.PORTGROUP_NAME
        network = model.Network(
            name=pg_name, network_type=constants.NETWORK_VLAN)
        vswitch = model.VirtualSwitch("test_dvs", hosts=None)
        self.assertFalse(self.vc_driver.delete_network(network, vswitch))

    def test_update_port(self):
        port_id = "PORT-1234-5678"
        vm_id = fake_vmware_api.Constants.VM_UUID
        mac_address = fake_vmware_api.Constants.VM_MAC
        vlan = model.Vlan(vlanIds=["1001"])
        network_config = model.NetworkConfig(vlan)
        network = model.Network(
            name="net-1234", network_type=constants.NETWORK_VLAN,
            config=network_config)
        port = model.Port(uuid=port_id,
                          name=None,
                          mac_address=mac_address,
                          ipaddresses=None,
                          vm_id=vm_id,
                          port_status=constants.PORT_STATUS_DOWN)
        self.vc_driver.update_port(network, port, None)
        self.assertTrue(fake_vmware_api.is_task_done(
            "ReconfigureDVPort_Task"))

    def test_update_port_up(self):
        port_id = "PORT-1234-5678"
        vm_id = fake_vmware_api.Constants.VM_UUID
        mac_address = fake_vmware_api.Constants.VM_MAC
        vlan = model.Vlan(vlanIds=["1001"])
        network_config = model.NetworkConfig(vlan)
        network = model.Network(
            name="net-1234", network_type=constants.NETWORK_VLAN,
            config=network_config)
        port = model.Port(uuid=port_id,
                          name=None,
                          mac_address=mac_address,
                          ipaddresses=None,
                          vm_id=vm_id,
                          port_status=constants.PORT_STATUS_UP)
        self.vc_driver.update_port(network, port, None)
        self.assertTrue(fake_vmware_api.is_task_done(
            "ReconfigureDVPort_Task"))

    def test_update_port_invalid_status(self):
        port_id = "PORT-1234-5678"
        vm_id = fake_vmware_api.Constants.VM_UUID
        mac_address = fake_vmware_api.Constants.VM_MAC
        vlan = model.Vlan(vlanIds=["1001"])
        network_config = model.NetworkConfig(vlan)
        network = model.Network(
            name="net-1234", network_type=constants.NETWORK_VLAN,
            config=network_config)
        port = model.Port(uuid=port_id,
                          name=None,
                          mac_address=mac_address,
                          ipaddresses=None,
                          vm_id=vm_id,
                          port_status="Invalid")
        self.assertRaises(error.OVSvAppNeutronAgentError,
                          self.vc_driver.update_port,
                          network,
                          port,
                          None)

    def test_update_port_invalid_vm(self):
        port_id = "PORT-1234-5678"
        vm_id = "INVALID-VM"
        mac_address = fake_vmware_api.Constants.VM_MAC
        vlan = model.Vlan(vlanIds=["1001"])
        network_config = model.NetworkConfig(vlan)
        network = model.Network(
            name="net-1234", network_type=constants.NETWORK_VLAN,
            config=network_config)
        port = model.Port(uuid=port_id,
                          name=None,
                          mac_address=mac_address,
                          ipaddresses=None,
                          vm_id=vm_id,
                          port_status=constants.PORT_STATUS_UP)
        done = self.vc_driver.update_port(network, port, None)
        self.assertFalse(done)
        self.assertFalse(fake_vmware_api.is_task_done(
            "ReconfigureDVPort_Task"))

    def test_update_port_invalid_mac(self):
        port_id = "PORT-1234-5678"
        vm_id = fake_vmware_api.Constants.VM_UUID
        mac_address = "in:va:li:d0:ma:c0"
        vlan = model.Vlan(vlanIds=["1001"])
        network_config = model.NetworkConfig(vlan)
        network = model.Network(
            name="net-1234", network_type=constants.NETWORK_VLAN,
            config=network_config)
        port = model.Port(uuid=port_id,
                          name=None,
                          mac_address=mac_address,
                          ipaddresses=None,
                          vm_id=vm_id,
                          port_status=constants.PORT_STATUS_UP)
        self.vc_driver.update_port(network, port, None)
        self.assertFalse(fake_vmware_api.is_task_done(
            "ReconfigureDVPort_Task"))

    def test_post_create_port(self):
        vm_id = fake_vmware_api.Constants.VM_UUID
        network_uuid = fake_vmware_api.Constants.PORTGROUP_NAME
        port = model.Port(name=None,
                          mac_address=None,
                          ipaddresses=None,
                          vm_id=vm_id,
                          port_status=constants.PORT_STATUS_UP,
                          network_uuid=network_uuid)
        self.vc_driver.post_create_port(port)
        self.assertTrue(fake_vmware_api.is_task_done(
            "ReconfigureDVPort_Task"))

    def test_post_create_port_status_down(self):
        vm_id = fake_vmware_api.Constants.VM_UUID
        network_uuid = fake_vmware_api.Constants.PORTGROUP_NAME
        port = model.Port(name=None,
                          mac_address=None,
                          ipaddresses=None,
                          vm_id=vm_id,
                          port_status=constants.PORT_STATUS_DOWN,
                          network_uuid=network_uuid)
        self.vc_driver.post_create_port(port)
        self.assertFalse(fake_vmware_api.is_task_done(
            "ReconfigureDVPort_Task"))

    def test_wait_for_port_update_on_vm_exception(self):
        vm_mor = None
        pg_mor = None
        try:
            with contextlib.nested(
                mock.path.object(self.vc_driver,
                                 '_register_vm_for_updates'),
                mock.patch.object(vim_util, "wait_for_updates_ex",
                                  side_effect=Exception())):
                    self.vc_driver._wait_for_port_update_on_vm(vm_mor, pg_mor)
        except Exception:
            self.assertFalse(fake_vmware_api.is_task_done(
                "ReconfigureDVPort_Task"))

    def test_post_create_port_vm_deleted(self):
        vm_id = fake_vmware_api.Constants.VM_UUID
        network_uuid = fake_vmware_api.Constants.PORTGROUP_NAME
        port = model.Port(name=None,
                          mac_address=None,
                          ipaddresses=None,
                          vm_id=vm_id,
                          port_status=constants.PORT_STATUS_UP,
                          network_uuid=network_uuid)
        updateSet = fake_vmware_api.DataObject()
        updateSet.version = 1
        filterSet = []
        updateSet.filterSet = filterSet
        propFilterUpdate = fake_vmware_api.DataObject()
        filterSet.append(propFilterUpdate)
        objectSet = []
        propFilterUpdate.objectSet = objectSet
        objectUpdate = fake_vmware_api.DataObject()
        objectUpdate.kind = "leave"
        objectSet.append(objectUpdate)
        with mock.patch.object(vim_util, "wait_for_updates_ex",
                               return_value=updateSet) as wait_update:
            self.vc_driver.post_create_port(port)
            self.assertTrue(wait_update.called)

    def test_post_create_port_nic_devices_none(self):
        try:
            vm_id = fake_vmware_api.Constants.VM_UUID
            network_uuid = fake_vmware_api.Constants.PORTGROUP_NAME
            port = model.Port(name=None,
                              mac_address=None,
                              ipaddresses=None,
                              vm_id=vm_id,
                              port_status=constants.PORT_STATUS_UP,
                              network_uuid=network_uuid)
            updateSet = fake_vmware_api.DataObject()
            updateSet.version = 1
            filterSet = []
            updateSet.filterSet = filterSet
            propFilterUpdate = fake_vmware_api.DataObject()
            filterSet.append(propFilterUpdate)
            objectSet = []
            propFilterUpdate.objectSet = objectSet
            objectUpdate = fake_vmware_api.DataObject()
            objectSet.append(objectUpdate)
            with mock.patch.object(vim_util, "wait_for_updates_ex",
                                   return_value=updateSet):
                self.vc_driver.post_create_port(port)
        except Exception:
            self.assertFalse(fake_vmware_api.
                             is_task_done("ReconfigureDVPort_Task"))

    def test_post_create_port_excp1(self):
        vm_id = fake_vmware_api.Constants.VM_UUID
        network_uuid = fake_vmware_api.Constants.PORTGROUP_NAME
        port = model.Port(name=None,
                          mac_address=None,
                          ipaddresses=None,
                          vm_id=vm_id,
                          port_status=constants.PORT_STATUS_UP,
                          network_uuid=network_uuid)
        with mock.patch.object(network_util, "get_portgroup_mor_by_name",
                               return_value=None):
            self.assertRaises(error_util.RunTimeError,
                              self.vc_driver.post_create_port,
                              port)
            self.assertFalse(fake_vmware_api.is_task_done(
                "ReconfigureDVPort_Task"))

    def test_post_create_port_excp2(self):
        vm_id = fake_vmware_api.Constants.VM_UUID
        network_uuid = fake_vmware_api.Constants.PORTGROUP_NAME
        port = model.Port(name=None,
                          mac_address=None,
                          ipaddresses=None,
                          vm_id=vm_id,
                          port_status=constants.PORT_STATUS_UP,
                          network_uuid=network_uuid)
        with contextlib.nested(
            mock.patch.object(dvs_driver.DvsNetworkDriver,
                              "_find_cluster_switch_for_vm",
                              return_value=[None, None, "test_vds"]),
            mock.patch.object(network_util, "get_portgroup_mor_by_name",
                              return_value="pg_mor"),
            mock.patch.object(resource_util, "get_vm_mor_for_uuid",
                              return_value=None)):
                self.assertRaises(error_util.RunTimeError,
                                  self.vc_driver.post_create_port,
                                  port)
                self.assertFalse(fake_vmware_api.is_task_done(
                    "ReconfigureDVPort_Task"))

    def test_post_create_port_excp3(self):
        vm_id = fake_vmware_api.Constants.VM_UUID
        network_uuid = fake_vmware_api.Constants.PORTGROUP_NAME
        port = model.Port(name=None,
                          mac_address=None,
                          ipaddresses=None,
                          vm_id=vm_id,
                          port_status=constants.PORT_STATUS_UP,
                          network_uuid=network_uuid)
        with mock.patch.object(dvs_driver.DvsNetworkDriver,
                               "_register_vm_for_updates",
                               side_effect=error_util.RunTimeError(
                                   "Exception in registering vm for updates")):
            self.assertRaises(error_util.RunTimeError,
                              self.vc_driver.post_create_port,
                              port)
            self.assertFalse(fake_vmware_api.is_task_done(
                "ReconfigureDVPort_Task"))


# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import eventlet
eventlet.monkey_patch()
from oslo_log import log
from oslo_utils import timeutils

from neutron.common import rpc as n_rpc
from neutron.common import topics
from neutron import context as neutron_context
from neutron.extensions import portbindings
from neutron import manager
from neutron.plugins.common import constants as p_const
from neutron.plugins.ml2 import driver_api as api

from networking_vsphere.common import constants
from networking_vsphere.db import ovsvapp_db
from networking_vsphere.ml2 import ovsvapp_rpc

LOG = log.getLogger(__name__)


class OVSvAppAgentDriver(object):
    """OVSvApp Python Driver for Neutron.

    This code is the backend implementation for the OVSvApp ML2
    MechanismDriver for OpenStack Neutron.
    """

    def initialize(self):
        self.context = neutron_context.get_admin_context()
        self._start_rpc_listeners()
        self._plugin = None
        self._pool = None

    @property
    def plugin(self):
        if self._plugin is None:
            self._plugin = manager.NeutronManager.get_plugin()
        return self._plugin

    @property
    def threadpool(self):
        if self._pool is None:
            self._pool = eventlet.GreenPool(2)
        return self._pool

    def _start_rpc_listeners(self):
        self.notifier = ovsvapp_rpc.OVSvAppAgentNotifyAPI(topics.AGENT)
        self.endpoints = [ovsvapp_rpc.OVSvAppServerRpcCallback(self.notifier)]
        self.topic = constants.OVSVAPP
        self.conn = n_rpc.create_connection(new=True)
        self.conn.create_consumer(self.topic, self.endpoints, fanout=False)
        return self.conn.consume_in_threads()

    def _get_ovsvapp_agent_from_cluster(self, context, cluster_id):
        chosen_agent = None
        agents = self.plugin.get_agents(
            context,
            filters={'agent_type': [constants.AGENT_TYPE_OVSVAPP]})
        chosen_agent = agents[0]
        recent_time = chosen_agent['heartbeat_timestamp']
        agents = agents[1:]
        for agent in agents:
            agent_cluster_id = agent['configurations'].get('cluster_id')
            if cluster_id != agent_cluster_id:
                continue
            delta = timeutils.delta_seconds(recent_time,
                                            agent['heartbeat_timestamp'])
            if delta > 0:
                recent_time = agent['heartbeat_timestamp']
                chosen_agent = agent
        return chosen_agent

    def _notify_agent(self, network_info):
        host = None
        if 'host' in network_info:
            host = network_info['host']
        else:
            cluster = network_info['cluster_id']
            agent = self._get_ovsvapp_agent_from_cluster(self.context,
                                                         cluster)
            if agent and 'host' in agent:
                host = agent['host']
            else:
                LOG.error(_("Failed to find OVSvApp Agent with host "
                            "%(host)s while releasing network allocations "
                            "for %(cluster)s in vCenter %(vcenter)s."),
                          {'host': host,
                           'vcenter': network_info['vcenter_id'],
                           'cluster': cluster})
                return
        try:
            self.notifier.device_delete(self.context, network_info, host)
        except Exception:
            LOG.exception(_("Failed to notify agent to delete port group "))

    def delete_port_postcommit(self, context):
        """Delete port non-database commit event."""
        port = context.current
        if port and port['device_owner'].startswith('compute'):
            segment = context.bound_segment
            if segment and segment[api.NETWORK_TYPE] == p_const.TYPE_VXLAN:
                host = port[portbindings.HOST_ID]
                vcenter = None
                cluster = None
                agents = self.plugin.get_agents(
                    self.context,
                    filters={'agent_type': [constants.AGENT_TYPE_OVSVAPP],
                             'host': [host]})
                agent = agents[0]
                vcenter = agent['configurations']['vcenter']
                cluster = agent['configurations']['cluster_id']
                net_info = {'vcenter_id': vcenter,
                            'cluster_id': cluster,
                            'network_id': port['network_id'],
                            'segmentation_id': segment[api.SEGMENTATION_ID],
                            'host': host}
                try:
                    lvid = ovsvapp_db.check_to_reclaim_local_vlan(net_info)
                    if lvid >= 1:
                        net_info.update({'lvid': lvid})
                        LOG.debug("Spawning thread for releasing network "
                                  "VNI allocations for %s.", net_info)
                        self.threadpool.spawn_n(self._notify_agent, net_info)
                except Exception:
                    LOG.exception(_("Failed to check for reclaiming "
                                    "local vlan."))

    def delete_network_postcommit(self, context):
        try:
            network = context.current
            segments = context.network_segments
            vxlan_segments = []
            if segments:
                for segment in segments:
                    if segment[api.NETWORK_TYPE] == p_const.TYPE_VXLAN:
                        vxlan_segments.append(segment)
            if not vxlan_segments:
                return
            stale_entries = ovsvapp_db.get_stale_local_vlans_for_network(
                network['id'])
            if stale_entries:
                for (vcenter, cluster, lvid) in stale_entries:
                    network_info = {'vcenter_id': vcenter,
                                    'cluster_id': cluster,
                                    'lvid': lvid,
                                    'network_id': network['id']}
                    if len(vxlan_segments) == 1:
                        seg_id = vxlan_segments[0][api.SEGMENTATION_ID]
                        network_info.update({'segmentation_id': seg_id})
                    LOG.debug("Spawning thread for releasing network "
                              "VNI allocations for %s.", network_info)
                    self.threadpool.spawn_n(self._notify_agent, network_info)
        except Exception:
            LOG.exception(_("Failed to check for stale "
                            "local vlan allocations."))



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import signal
import sys

import eventlet
eventlet.monkey_patch()
from oslo_config import cfg
from oslo_log import log

from networking_vsphere.agent import agent
from networking_vsphere.common import config as ovsvapp_config
from networking_vsphere.common import utils

LOG = log.getLogger(__name__)

agent_obj = None


def run():
    signal.signal(signal.SIGTERM, signal_handler)
    signal.signal(signal.SIGINT, signal_handler)
    try:
        global agent_obj
        ovsvapp_config.register_options()
        LOG.info(_("Loading agent: %s."), cfg.CONF.OVSVAPP.agent_driver)
        agent_obj = utils.load_object(cfg.CONF.OVSVAPP.agent_driver,
                                      agent.Agent)
        agent_obj.start()
    except Exception as e:
        LOG.exception(_("Error in L2 agent service."))
        if agent_obj:
            agent_obj.stop()
        sys.exit(_("ERROR: %s.") % e)


def signal_handler(signum, frame):
    signals_to_names = {}
    for n in dir(signal):
        if n.startswith('SIG') and not n.startswith('SIG_'):
            signals_to_names[getattr(signal, n)] = n
    LOG.info(_("Caught %s, exiting."), signals_to_names[signum])
    if agent_obj:
        try:
            agent_obj.stop()
        except Exception:
            # Ignore any exceptions while exiting.
            pass
    signal.signal(signum, signal.SIG_DFL)
    sys.exit(0)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from networking_vsphere.tests import base
from networking_vsphere.tests.unit.utils import fake_vmware_api
from networking_vsphere.tests.unit.utils import stubs
from networking_vsphere.utils import resource_util


class TestVmwareResourceUtil(base.TestCase):

    def setUp(self):
        super(TestVmwareResourceUtil, self).setUp()
        self.fake_visdk = self.useFixture(stubs.FakeVmware())
        self.session = self.fake_visdk.session
        self.useFixture(stubs.CacheFixture())

    def test_get_host_mor_for_vm(self):
        host_mor = resource_util.get_host_mor_for_vm(
            self.session, fake_vmware_api.Constants.VM_UUID)
        self.assertTrue(host_mor)

    def test_get_host_mor_for_invalid_vm(self):
        host_mor = resource_util.get_host_mor_for_vm(
            self.session, "1234-1234-1234-1234")
        self.assertFalse(host_mor)

    def test_get_host_mor_by_name(self):
        host_mor = resource_util.get_host_mor_by_name(
            self.session, fake_vmware_api.Constants.HOST_NAME)
        self.assertTrue(host_mor)

    def test_get_host_mor_for_invalid_hostname(self):
        host_mor = resource_util.get_host_mor_by_name(
            self.session, "fake_host")
        self.assertFalse(host_mor)

    def test_get_hostname_for_host_mor(self):
        host_mor = resource_util.get_host_mor_by_name(
            self.session, fake_vmware_api.Constants.HOST_NAME)
        host_name = resource_util.get_hostname_for_host_mor(
            self.session, host_mor)
        self.assertTrue(host_name)

    def test_get_vm_mor_by_name(self):
        vm_mor = resource_util.get_vm_mor_by_name(
            self.session, fake_vmware_api.Constants.VM_NAME)
        self.assertTrue(vm_mor)

    def test_get_cluster_mor_for_vm(self):
        cluster_mor = resource_util.get_cluster_mor_for_vm(
            self.session, fake_vmware_api.Constants.VM_UUID)
        self.assertTrue(cluster_mor)

    def test_get_cluster_mor_for_invalid_vm(self):
        cluster_mor = resource_util.get_cluster_mor_for_vm(
            self.session, "1234-1234-1234-1234")
        self.assertFalse(cluster_mor)

    def test_get_vm_mor_for_uuid(self):
        vm_mor = resource_util.get_vm_mor_for_uuid(
            self.session, fake_vmware_api.Constants.VM_UUID)
        self.assertTrue(vm_mor)

    def test_get_vm_mor_for_invalid_uuid(self):
        vm_mor = resource_util.get_vm_mor_for_uuid(
            self.session, "1234-1234-1234-1234")
        self.assertFalse(vm_mor)

    def test_get_host_mors_for_cluster(self):
        cluster_mor = resource_util.get_cluster_mor_for_vm(
            self.session, fake_vmware_api.Constants.VM_UUID)
        self.assertTrue(cluster_mor)
        host_mor = resource_util.get_host_mors_for_cluster(
            self.session, cluster_mor)
        self.assertTrue(host_mor)
        self.assertTrue(isinstance(host_mor, list))

    def test_get_host_mors_for_cluster_with_invalid_mor(self):
        host_mor = resource_util.get_host_mors_for_cluster(self.session, None)
        self.assertFalse(host_mor)

    def test_get_extraconfigs_for_vm(self):
        vm_mor = resource_util.get_vm_mor_for_uuid(
            self.session, fake_vmware_api.Constants.VM_UUID)
        self.assertTrue(vm_mor)
        extraconfigs = resource_util.get_extraconfigs_for_vm(
            self.session, vm_mor)
        self.assertTrue(extraconfigs)

    def tearDown(self):
        base.TestCase.tearDown(self)


# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

import contextlib
from neutron.extensions import portbindings
from neutron.plugins.common import constants as p_const
from neutron.plugins.ml2 import driver_api as api
from neutron.tests.unit.plugins.ml2 import _test_mech_agent as base

from networking_vsphere.ml2 import ovsvapp_driver

network = {'id': 'net_id'}

net_info = {'vcenter_id': 'fake_vcenter',
            'cluster_id': 'fake_cluster',
            'network_id': 'net_id',
            'segmentation_id': 1234,
            'lvid': 1}

net_info_with_host = {'vcenter_id': 'fake_vcenter',
                      'cluster_id': 'fake_cluster',
                      'network_id': 'net_id',
                      'host': 'fake_host',
                      'segmentation_id': 1234,
                      'lvid': 1}

fake_agent = {'configurations': {'vcenter': 'fake_vcenter',
                                 'cluster_id': 'fake_cluster'},
              'host': 'fake_host'}

compute_port = {'id': 'fake_id',
                'device_owner': 'compute-nova',
                'network_id': 'net_id',
                portbindings.HOST_ID: 'fake_host'}

dhcp_port = {'id': 'fake_id',
             'device_owner': 'network_dhcp',
             'network_id': 'net_id',
             portbindings.HOST_ID: 'fake_host'}

vlan_segment = {api.NETWORK_TYPE: p_const.TYPE_VLAN}

vxlan_segment = {api.NETWORK_TYPE: p_const.TYPE_VXLAN,
                 api.SEGMENTATION_ID: 1234}


class FakeContext():

    def __init__(self, current, bound_segment):
        self.current = current
        self.bound_segment = bound_segment

    @property
    def network_segments(self):
        return [self.bound_segment]


class FakePlugin():

    def get_agents(self):
        return


class OVSvAppAgentDriverTestCase(base.AgentMechanismBaseTestCase):

    def setUp(self):
        super(OVSvAppAgentDriverTestCase, self).setUp()
        self.driver = ovsvapp_driver.OVSvAppAgentDriver()
        self.driver.initialize()
        self.driver._plugin = FakePlugin()

    def test_notify_agent_without_host(self):
        with contextlib.nested(
            mock.patch.object(self.driver,
                              '_get_ovsvapp_agent_from_cluster',
                              return_value=fake_agent),
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'check_to_reclaim_local_vlan', return_value=1),
            mock.patch.object(self.driver.notifier,
                              'device_delete', return_value=True),
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'release_local_vlan'),
        ) as (get_agent, check_to_reclaim_local_vlan, device_delete_rpc,
              release_local_vlan):
            self.driver._notify_agent(net_info)
            self.assertTrue(get_agent.called)
            self.assertFalse(check_to_reclaim_local_vlan.called)
            self.assertTrue(device_delete_rpc.called)
            self.assertFalse(release_local_vlan.called)

    def test_notify_agent_without_host_no_agent(self):
        with contextlib.nested(
            mock.patch.object(self.driver,
                              '_get_ovsvapp_agent_from_cluster',
                              return_value=None),
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'check_to_reclaim_local_vlan'),
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'release_local_vlan'),
        ) as (get_agent, check_to_reclaim_local_vlan, release_local_vlan):
            self.driver._notify_agent(net_info)
            self.assertTrue(get_agent.called)
            self.assertFalse(check_to_reclaim_local_vlan.called)
            self.assertFalse(release_local_vlan.called)

    def test_notify_agent_without_host_rpc_failed(self):
        with contextlib.nested(
            mock.patch.object(self.driver,
                              '_get_ovsvapp_agent_from_cluster',
                              return_value=fake_agent),
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'check_to_reclaim_local_vlan', return_value=1),
            mock.patch.object(self.driver.notifier,
                              'device_delete', return_value=False),
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'release_local_vlan'),
        ) as (get_agent, check_to_reclaim_local_vlan, device_delete_rpc,
              release_local_vlan):
            self.driver._notify_agent(net_info)
            self.assertTrue(get_agent.called)
            self.assertFalse(check_to_reclaim_local_vlan.called)
            self.assertTrue(device_delete_rpc.called)
            self.assertFalse(release_local_vlan.called)

    def test_notify_agent_with_host(self):
        with contextlib.nested(
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'check_to_reclaim_local_vlan', return_value=1),
            mock.patch.object(self.driver.notifier,
                              'device_delete', return_value=True),
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'release_local_vlan'),
        ) as (check_to_reclaim_local_vlan, device_delete_rpc,
              release_local_vlan):
            self.driver._notify_agent(net_info_with_host)
            self.assertFalse(check_to_reclaim_local_vlan.called)
            self.assertTrue(device_delete_rpc.called)
            self.assertFalse(release_local_vlan.called)

    def test_delete_port_postcommit_vlan_port(self):
        port_context = FakeContext(compute_port, vlan_segment)
        with contextlib.nested(
            mock.patch.object(self.driver._plugin, 'get_agents'),
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'check_to_reclaim_local_vlan', return_value=1),
            mock.patch('eventlet.GreenPool.spawn_n')
        ) as (get_agents, check_to_reclaim_local_vlan, spawn_thread):
            self.driver.delete_port_postcommit(port_context)
            self.assertFalse(get_agents.called)
            self.assertFalse(check_to_reclaim_local_vlan.called)
            self.assertFalse(spawn_thread.called)

    def test_delete_port_postcommit_dhcp_port(self):
        port_context = FakeContext(dhcp_port, vlan_segment)
        with contextlib.nested(
            mock.patch.object(self.driver._plugin, 'get_agents',
                              return_value=[fake_agent]),
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'check_to_reclaim_local_vlan', return_value=1),
            mock.patch('eventlet.GreenPool.spawn_n')
        ) as (get_agents, check_to_reclaim_local_vlan, spawn_thread):
            self.driver.delete_port_postcommit(port_context)
            self.assertFalse(get_agents.called)
            self.assertFalse(check_to_reclaim_local_vlan.called)
            self.assertFalse(spawn_thread.called)

    def test_delete_port_postcommit_vxlan_port_release_not_required(self):
        port_context = FakeContext(compute_port, vxlan_segment)
        with contextlib.nested(
            mock.patch.object(self.driver._plugin, 'get_agents',
                              return_value=[fake_agent]),
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'check_to_reclaim_local_vlan', return_value=-1),
            mock.patch('eventlet.GreenPool.spawn_n')
        ) as (get_agents, check_to_reclaim_local_vlan, spawn_thread):
            self.driver.delete_port_postcommit(port_context)
            self.assertTrue(get_agents.called)
            self.assertTrue(check_to_reclaim_local_vlan.called)
            self.assertFalse(spawn_thread.called)

    def test_delete_port_postcommit_vxlan_port_release_required(self):
        port_context = FakeContext(compute_port, vxlan_segment)
        with contextlib.nested(
            mock.patch.object(self.driver._plugin, 'get_agents',
                              return_value=[fake_agent]),
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'check_to_reclaim_local_vlan', return_value=True),
            mock.patch('eventlet.GreenPool.spawn_n')
        ) as (get_agents, check_to_reclaim_local_vlan, spawn_thread):
            self.driver.delete_port_postcommit(port_context)
            self.assertTrue(get_agents.called)
            self.assertTrue(check_to_reclaim_local_vlan.called)
            self.assertTrue(spawn_thread.called)
            spawn_thread.assert_called_with(
                self.driver._notify_agent, net_info_with_host)

    def test_delete_network_postcommit_no_stale(self):
        net_context = FakeContext(network, vxlan_segment)
        with contextlib.nested(
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'get_stale_local_vlans_for_network', return_value=[]),
            mock.patch('eventlet.GreenPool.spawn_n')
        ) as (get_stale_entries, spawn_thread):
            self.driver.delete_network_postcommit(net_context)
            self.assertTrue(get_stale_entries.called)
            get_stale_entries.assert_called_with('net_id')
            self.assertFalse(spawn_thread.called)

    def test_delete_network_postcommit(self):
        net_context = FakeContext(network, vxlan_segment)
        ret_val = [('fake_vcenter', 'fake_cluster_1', 1),
                   ('fake_vcenter', 'fake_cluster', 1)]
        with contextlib.nested(
            mock.patch('networking_vsphere.db.ovsvapp_db.'
                       'get_stale_local_vlans_for_network',
                       return_value=ret_val),
            mock.patch('eventlet.GreenPool.spawn_n')
        ) as (get_stale_entries, spawn_thread):
            self.driver.delete_network_postcommit(net_context)
            self.assertTrue(get_stale_entries.called)
            get_stale_entries.assert_called_with('net_id')
            self.assertTrue(spawn_thread.called)
            self.assertEqual(2, spawn_thread.call_count)
            spawn_thread.assert_called_with(
                self.driver._notify_agent, net_info)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import eventlet
import greenlet
from oslo_config import cfg
from oslo_log import log

from networking_vsphere.drivers import base_manager
from networking_vsphere.drivers import dvs_driver
from networking_vsphere.utils import vim_session

LOG = log.getLogger(__name__)


class VcenterManager(base_manager.DriverManager):

    def __init__(self, netcallback):
        base_manager.DriverManager.__init__(self)
        self.netcallback = netcallback
        self.vcenter_ip = None
        self.vcenter_username = None
        self.vcenter_password = None
        self.vcenter_api_retry_count = None
        self.wsdl_location = None
        self.cluster_switch_mapping = {}
        self.connection_thread = None
        self.https_port = cfg.CONF.VMWARE.https_port

    def _parse_mapping(self, entry):
        """Parse an entry of cluster_dvs_mapping.

        :param entry: String value which is an entry in conf file
                      for cluster_dvs_mapping.
                      Could be a simple mapping in the form
                      clusterpath:dvsname or a comma separated one like
                      clusterpath1:dvsname1,clusterpath2:dvsname2
        :returns: A list of (cluster, dvs) tuples
        """
        try:
            cluster_dvs_list = []
            LOG.debug("Parsing cluster_dvs_mapping %s." % entry)
            mappings = entry.split(",")
            for mapping in mappings:
                cluster = None
                vds = None
                if ":" in mapping:
                    cluster, vds = mapping.split(":", 1)
                    cluster = cluster.strip()
                    vds = vds.strip()
                if not cluster or not vds:
                    LOG.error(_("Invalid value %s for opt "
                                "cluster_dvs_mapping.") % mapping)
                else:
                    cluster_dvs_list.append((cluster, vds))
        except Exception:
            LOG.exception(_("Invalid value %s for opt cluster_dvs_mapping.")
                          % entry)
        return cluster_dvs_list

    def _add_cluster(self, cluster, vds):
        try:
            self.driver.add_cluster(cluster, vds)
        except Exception:
            LOG.exception(_("Adding cluster %(cluster)s:%(vds)s failed."),
                          {'cluster': cluster, 'vds': vds})
        else:
            self.cluster_switch_mapping[cluster] = vds

    def initialize_driver(self):
        self.stop()
        self.driver = None
        self.vcenter_ip = cfg.CONF.VMWARE.vcenter_ip
        self.vcenter_username = cfg.CONF.VMWARE.vcenter_username
        self.vcenter_password = cfg.CONF.VMWARE.vcenter_password
        self.vcenter_api_retry_count = cfg.CONF.VMWARE.vcenter_api_retry_count
        self.wsdl_location = cfg.CONF.VMWARE.wsdl_location
        self.https_port = cfg.CONF.VMWARE.https_port
        if (self.vcenter_ip and self.vcenter_username and
                self.vcenter_password and self.wsdl_location):
            vim_session.ConnectionHandler.set_vc_details(
                self.vcenter_ip,
                self.vcenter_username,
                self.vcenter_password,
                self.vcenter_api_retry_count,
                self.wsdl_location,
                self.https_port)
            vim_session.ConnectionHandler.start()
            if self.connection_thread:
                self.connection_thread.kill()
            self.connection_thread = eventlet.spawn(
                vim_session.ConnectionHandler.try_connection)
            try:
                self.connection_thread.wait()
            except greenlet.GreenletExit:
                LOG.warn(_("Thread waiting on vCenter connection exited."))
                return
        else:
            LOG.error(_("Must specify vcenter_ip, vcenter_username, "
                        "vcenter_password and wsdl_location."))
            return
        self.driver = dvs_driver.DvsNetworkDriver()
        self.driver.set_callback(self.netcallback)
        for mapping in cfg.CONF.VMWARE.cluster_dvs_mapping:
            cluster_dvs_list = self._parse_mapping(mapping)
            for cluster, vds in cluster_dvs_list:
                self._add_cluster(cluster, vds)

    def start(self):
        """Start the driver event monitoring."""
        if self.driver:
            eventlet.spawn_n(self.driver.monitor_events)

    def pause(self):
        """Pause the driver."""
        if self.driver:
            self.driver.pause()

    def stop(self):
        """Stop driver and connection."""
        if self.driver:
            self.driver.stop()
        vim_session.ConnectionHandler.stop()



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from networking_vsphere.common import error
from networking_vsphere.common import utils
from networking_vsphere.drivers import driver
from networking_vsphere.tests import base
from networking_vsphere.tests.unit.drivers import fake_driver


class CommonUtilsTestCase(base.TestCase):

    def setUp(self):
        super(CommonUtilsTestCase, self).setUp()
        driver_path = "networking_vsphere.tests.unit.drivers.fake_driver."
        valid_driver = "FakeNetworkDriver"
        invalid_driver = "FakeInvalidDriver"
        self.fake_network_driver = driver_path + valid_driver
        self.fake_invalid_driver = driver_path + invalid_driver

    def test_import_class(self):
        class_obj = utils.import_class(self.fake_network_driver)
        self.assertTrue(class_obj == fake_driver.FakeNetworkDriver)

    def test_import_class_exc(self):
        import_class_fn = utils.import_class
        invalid_class = self.fake_network_driver + "Invalid"
        import_err = ImportError
        self.assertRaises(import_err, import_class_fn, invalid_class)

    def test_load_object(self):
        driver_obj = utils.load_object(
            self.fake_network_driver, driver.NetworkDriver)
        self.assertTrue(isinstance(driver_obj, fake_driver.FakeNetworkDriver))

    def test_non_loadable_object(self):
        load_object_fn = utils.load_object
        invalid_class = self.fake_invalid_driver
        type_err = TypeError
        self.assertRaises(type_err, load_object_fn,
                          invalid_class, driver.NetworkDriver)

    def test_fullname(self):

        class Sample(object):
            pass

        fake_sample = Sample()
        class_name = utils.fullname(fake_sample)
        result = "networking_vsphere.tests.unit.common.test_utils.Sample"
        self.assertTrue(class_name == result)

    def test_require_state(self):

        class TestClass:
            @utils.require_state(set(["RUNNING"]), True)
            def method_test(self):
                return "Success"

        obj = TestClass()
        obj.state = "RUNNING"
        self.assertTrue(obj.method_test() == "Success")

    def test_require_state_excp(self):

        class TestClass:
            @utils.require_state(set(["RUNNING"]), True)
            def method_test(self):
                return "Success"

        obj = TestClass()
        obj.state = "NOTRUNNING"
        self.assertRaises(error.OVSvAppNeutronAgentError, obj.method_test)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

import eventlet
import fixtures
import mock
from oslo_config import cfg
from oslotest import base

from networking_vsphere.common import config as ovsvapp_config

CONF = cfg.CONF
eventlet.monkey_patch()


class TestCase(base.BaseTestCase):

    """Test case base class for all unit tests."""

    def setUp(self):
        """Run before each test method to initialize test environment."""
        super(base.BaseTestCase, self).setUp()
        ovsvapp_config.register_options()
        self.mock = mock.Mock()
        self.logger = self.useFixture(fixtures.FakeLogger(name="neutron",
                                                          level=logging.INFO
                                                          ))
        self._overridden_opts = []
        self.addCleanup(self.del_attributes)
        self.addCleanup(self.reset_flags)

    def flags(self, **kw):
        """Override flag variables for a test."""
        group = kw.pop('group', None)
        module = kw.pop('module', None)
        for k, v in kw.iteritems():
            if module:
                CONF.import_opt(k, module, group)
            CONF.set_override(k, v, group)
            self._overridden_opts.append((k, group))

    def reset_flags(self):
        """Resets all flag variables for the test.

        Runs after each test.

        """
        for (k, group) in self._overridden_opts:
            CONF.clear_override(k, group)
        self._overridden_opts = []

    def del_attributes(self):
        """Deletes attributes for the test.

        Runs after each test.

        """
        # Delete attributes that don't start with _ so they don't pin
        # memory around unnecessarily for the duration of the test
        # suite
        for key in [k for k in self.__dict__.keys() if k[0] != '_']:
            del self.__dict__[key]



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


def convert_propset_to_dict(propset_list):
    propset_dict = {}
    for prop in propset_list:
        propset_dict[prop.name] = prop.val
    return propset_dict


def convert_objectupdate_to_dict(objectUpdate):
    changes = {}
    if (hasattr(objectUpdate, "changeSet")
            and objectUpdate.changeSet):
        for prop in objectUpdate.changeSet:
            if hasattr(prop, "name"):
                changes[prop.name] = None
                if hasattr(prop, "val"):
                    changes[prop.name] = prop.val
    return changes



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from neutron.tests.unit import testlib_api

from networking_vsphere.db import ovsvapp_db


class OVSvAppDBTestCase(testlib_api.SqlTestCase):

    def setUp(self):
        super(OVSvAppDBTestCase, self).setUp()

    def _form_port_info_dict(self, vcenter, cluster, network):
        return {'vcenter_id': vcenter,
                'cluster_id': cluster,
                'network_id': network,
                'port_id': 'fake_port'}

    def test_get_local_vlan_first_network_first_port(self):
        net1_port1_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net1')
        lvid = ovsvapp_db.get_local_vlan(net1_port1_info)
        self.assertEqual(1, lvid)

    def test_get_local_vlan_second_network_first_port(self):
        net1_port1_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net1')
        ovsvapp_db.get_local_vlan(net1_port1_info)

        net2_port1_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net2')

        lvid = ovsvapp_db.get_local_vlan(net2_port1_info)
        self.assertEqual(2, lvid)

    def test_get_local_vlan_first_network_second_port(self):
        net1_port_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net1')
        ovsvapp_db.get_local_vlan(net1_port_info)

        net2_port1_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net2')
        ovsvapp_db.get_local_vlan(net2_port1_info)

        lvid = ovsvapp_db.get_local_vlan(net1_port_info)
        self.assertEqual(1, lvid)

    def test_get_local_vlan_read_before_write(self):
        net1_port1_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net1')
        lvid = ovsvapp_db.get_local_vlan(net1_port1_info, False)
        self.assertIsNone(lvid)

    def test_get_local_vlan_write_and_read(self):
        net1_port1_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net1')
        ovsvapp_db.get_local_vlan(net1_port1_info)
        lvid = ovsvapp_db.get_local_vlan(net1_port1_info, False)
        self.assertEqual(1, lvid)

    def test_get_local_vlan_initialize_error(self):
        net1_port_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net1')
        with mock.patch('networking_vsphere.db.ovsvapp_db.'
                        '_initialize_lvids_for_cluster', return_value=False):
            lvid = ovsvapp_db.get_local_vlan(net1_port_info)
            self.assertIsNone(lvid)

    def test_check_to_reclaim_local_vlan(self):
        net1_port1_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net1')
        ovsvapp_db.get_local_vlan(net1_port1_info)

        lvid = ovsvapp_db.check_to_reclaim_local_vlan(net1_port1_info)
        self.assertEqual(1, lvid)

    def test_check_to_reclaim_local_vlan_multiple_ports(self):
        net1_port_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net1')
        ovsvapp_db.get_local_vlan(net1_port_info)
        ovsvapp_db.get_local_vlan(net1_port_info)
        ovsvapp_db.get_local_vlan(net1_port_info)

        lvid = ovsvapp_db.check_to_reclaim_local_vlan(net1_port_info)
        self.assertEqual(-1, lvid)
        lvid = ovsvapp_db.check_to_reclaim_local_vlan(net1_port_info)
        self.assertEqual(-1, lvid)
        lvid = ovsvapp_db.check_to_reclaim_local_vlan(net1_port_info)
        self.assertEqual(1, lvid)

    def test_release_local_vlan_already_released_network(self):
        net1_port_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net1')
        with mock.patch('networking_vsphere.db.ovsvapp_db.'
                        'LOG.error') as error_log:
            ovsvapp_db.release_local_vlan(net1_port_info)
            self.assertTrue(error_log.called)

    def test_release_local_vlan(self):
        net1_port_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net1')
        # Allocate network.
        ovsvapp_db.get_local_vlan(net1_port_info)

        # Setup for release.
        lvid = ovsvapp_db.check_to_reclaim_local_vlan(net1_port_info)
        self.assertEqual(1, lvid)
        with mock.patch('networking_vsphere.db.ovsvapp_db.'
                        'LOG.error') as error_log:
            ovsvapp_db.release_local_vlan(net1_port_info)
            self.assertFalse(error_log.called)

    def test_get_stale_local_vlans_for_network(self):
        net1_port1_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster1', 'net1')
        ovsvapp_db.get_local_vlan(net1_port1_info)

        net1_port2_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster2', 'net1')
        ovsvapp_db.get_local_vlan(net1_port2_info)
        lvid = ovsvapp_db.check_to_reclaim_local_vlan(net1_port1_info)
        self.assertEqual(1, lvid)
        lvid = ovsvapp_db.check_to_reclaim_local_vlan(net1_port2_info)
        self.assertEqual(1, lvid)
        ret_val = ovsvapp_db.get_stale_local_vlans_for_network('net1')
        self.assertEqual([('fake_vcenter', 'fake_cluster1', 1),
                          ('fake_vcenter', 'fake_cluster2', 1)], ret_val)

    def test_get_stale_local_vlans_for_network_no_stale_networks(self):
        net1_port_info = self._form_port_info_dict(
            'fake_vcenter', 'fake_cluster', 'net1')
        # Allocate network.
        ovsvapp_db.get_local_vlan(net1_port_info)

        # Setup for release.
        lvid = ovsvapp_db.check_to_reclaim_local_vlan(net1_port_info)
        self.assertEqual(1, lvid)
        ovsvapp_db.release_local_vlan(net1_port_info)
        ret_val = ovsvapp_db.get_stale_local_vlans_for_network('net1')
        self.assertIsNone(ret_val)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from eventlet import greenthread
from oslo_config import cfg
from oslo_log import log

from nova import exception
from nova.virt.vmwareapi import driver as vmware_driver
from nova.virt.vmwareapi import images
from nova.virt.vmwareapi import vim_util
from nova.virt.vmwareapi import vm_util

LOG = log.getLogger(__name__)

vmware_neutron_opts = [
    cfg.IntOpt('vmwareapi_nic_attach_retry_count',
               default=25,
               help='The number of times we retry to '
                    'attach nic on a portgroup.')]

CONF = cfg.CONF
CONF.register_opts(vmware_neutron_opts, 'vmware')


class OVSvAppVCDriver(vmware_driver.VMwareVCDriver):
    def __init__(self, virtapi):
        super(OVSvAppVCDriver, self).__init__(virtapi)
        self.client_factory = self._session.vim.client.factory
        self.old_modified_time = -1

    def spawn(self, context, instance, image_meta, injected_files,
              admin_password, network_info=None, block_device_info=None):
        _vmops = self._get_vmops_for_compute_node(instance['node'])
        _vmops.spawn(context=context,
                     instance=instance,
                     image_meta=image_meta,
                     injected_files=injected_files,
                     admin_password=admin_password,
                     network_info=None,
                     block_device_info=block_device_info,
                     power_on=False)
        vm_ref = vm_util.get_vm_ref(self._session, instance)
        if vm_ref is None:
            raise exception.InstanceNotFound(instance_id=instance['uuid'])
        image_info = images.VMwareImage.from_image(instance.image_ref,
                                                   image_meta)
        self._create_virtual_nic(instance, image_info, network_info, vm_ref)
        self._power_on_vm(instance, vm_ref)

    def _power_on_vm(self, instance, vm_ref):
        LOG.debug("Powering on the VM: %s.", instance)
        power_on_task = self._session._call_method(self._session.vim,
                                                   "PowerOnVM_Task", vm_ref)

        self._session._wait_for_task(power_on_task)
        LOG.debug("Powered on the VM: %s.", instance)

    def _get_mo_id_from_instance(self, instance):
        """Return the managed object ID from the instance.

        The instance['node'] will have the hypervisor_hostname field of the
        compute node on which the instance exists or will be provisioned.
        The name will be of the form:
            <mo id>.<vcenter uuid>
        e.g.
            domain-26.9d51f082-58a4-4449-beed-6fd205a5726b
        """
        return instance['node'].partition('.')[0]

    def _create_virtual_nic(self, instance, image_info, network_info, vm_ref):
        if network_info is None:
            return
        vif_model = image_info.vif_model
        if not vif_model:
            vif_model = "VirtualE1000"
        vif_infos = []
        for vif in network_info:
            portgroup_name = []
            mac_address = vif['address']
            network_id = vif['network']['id']
            portgroup_name.append(network_id)
            network_id_cluster_id = (network_id + "-" +
                                     self._get_mo_id_from_instance(instance))
            portgroup_name.append(network_id_cluster_id)
            # wait for port group creation (if not present) by neutron agent.
            network_ref = self._wait_and_get_portgroup_details(self._session,
                                                               vm_ref,
                                                               portgroup_name)
            if not network_ref:
                msg = ("Portgroup %(vlan)s (or) Portgroup %(vxlan)s.",
                       {'vlan': network_id, 'vxlan': network_id_cluster_id})
                raise exception.NetworkNotCreated(msg)
            vif_infos.append({
                             'network_name': network_id_cluster_id,
                             'mac_address': mac_address,
                             'network_ref': network_ref,
                             'iface_id': vif['id'],
                             'vif_model': vif_model
                             })

        config_spec = self.client_factory.create('ns0:'
                                                 'VirtualMachineConfigSpec')
        vif_spec_list = []
        for vif_info in vif_infos:
            vif_spec = vm_util._create_vif_spec(self.client_factory,
                                                vif_info)
            vif_spec_list.append(vif_spec)

        config_spec.deviceChange = vif_spec_list

        # add vm-uuid and iface-id.x values for Neutron.
        extra_config = []
        i = 0
        for vif_info in vif_infos:
            if vif_info['iface_id']:
                opt = self.client_factory.create('ns0:OptionValue')
                opt.key = "nvp.iface-id.%d" % i
                opt.value = vif_info['iface_id']
                extra_config.append(opt)
                i += 1

        config_spec.extraConfig = extra_config

        LOG.debug("Reconfiguring VM instance to attach NIC.")
        reconfig_task = self._session._call_method(self._session.vim,
                                                   "ReconfigVM_Task", vm_ref,
                                                   spec=config_spec)

        self._session._wait_for_task(reconfig_task)
        LOG.debug("Reconfigured VM instance to attach NIC.")

    def _wait_and_get_portgroup_details(self, session, vm_ref,
                                        port_group_name):
        """Gets reference to the portgroup for the vm."""

        max_counts = CONF.vmware.vmwareapi_nic_attach_retry_count
        count = 0
        network_obj = {}
        LOG.debug("Waiting for the portgroup %s to be created.",
                  port_group_name)
        while count < max_counts:
            host = session._call_method(vim_util, "get_dynamic_property",
                                        vm_ref, "VirtualMachine",
                                        "runtime.host")
            vm_networks_ret = session._call_method(vim_util,
                                                   "get_dynamic_property",
                                                   host, "HostSystem",
                                                   "network")
            if vm_networks_ret:
                vm_networks = vm_networks_ret.ManagedObjectReference
                for network in vm_networks:
                    # Get network properties.
                    if network._type == 'DistributedVirtualPortgroup':
                        props = session._call_method(vim_util,
                                                     "get_dynamic_property",
                                                     network,
                                                     network._type,
                                                     "config")
                        if props.name in port_group_name:
                            LOG.debug("DistributedVirtualPortgroup created.")
                            network_obj['type'] = 'DistributedVirtualPortgroup'
                            network_obj['dvpg'] = props.key
                            dvs_props = session._call_method(
                                vim_util,
                                "get_dynamic_property",
                                props.distributedVirtualSwitch,
                                "VmwareDistributedVirtualSwitch",
                                "uuid")
                            network_obj['dvsw'] = dvs_props
                            network_obj['dvpg-name'] = props.name
                            return network_obj
                    elif network._type == 'Network':
                        netname = session._call_method(vim_util,
                                                       "get_dynamic_property",
                                                       network,
                                                       network._type,
                                                       "name")
                        if netname in port_group_name:
                            LOG.debug("Standard Switch Portgroup created.")
                            network_obj['type'] = 'Network'
                            network_obj['name'] = port_group_name
                            return network_obj
                count = count + 1
                LOG.debug("Portgroup not created. Retrying again "
                          "after 2 seconds.")
                greenthread.sleep(2)
        return None



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

from neutron.plugins.common import constants as p_const

# vCenter server and ESX host related config read from ovsvapp_agent.ini.
VMWARE_OPTS = [
    cfg.StrOpt('vcenter_id',
               default=None,
               help='Unique ID of the vCenter Server on which this OVSvApp is'
                    'hosted'),
    cfg.StrOpt('vcenter_ip',
               default=None,
               help='vCenter server IP.'),
    cfg.StrOpt('vcenter_username',
               default=None,
               help='vCenter server user name.'),
    cfg.StrOpt('vcenter_password',
               default=None,
               help='vCenter server password.'),
    cfg.BoolOpt('cert_check',
                default=False,
                help='Enable SSL certificate check for vCenter.'),
    cfg.StrOpt('cert_path',
               default='/etc/ssl/certs/certs.pem',
               help='Certificate chain path containing cacert of vCenters.'),
    cfg.IntOpt('https_port',
               default=443,
               help='Customized https_port for vCenter communication.'),
    cfg.StrOpt('wsdl_location',
               default=None,
               help='vCenter server wsdl location.'),
    cfg.StrOpt('vcenter_api_retry_count',
               default=5,
               help='Number of retries while connecting to vcenter server.'),
    cfg.StrOpt('esx_hostname',
               default=None,
               help='ESX host name where this OVSvApp is hosted.'),
    cfg.BoolOpt('esx_maintenance_mode',
                default=True,
                help='Set host into maintenance mode.'),
    cfg.MultiStrOpt('cluster_dvs_mapping',
                    default=[''],
                    help='vCenter cluster to DVS mapping.')
]

# OVSvApp Agent related config read from ovsvapp_agent.ini and neutron.conf.
OVSVAPP_OPTS = [
    cfg.StrOpt('tenant_network_type',
               default=p_const.TYPE_VLAN,
               help='Network type for tenant networks'),
    cfg.StrOpt('integration_bridge',
               default='br-int',
               help='Integration Bridge.'),
    cfg.ListOpt('bridge_mappings',
                default=[],
                help='Bridge mappings.'),
    cfg.StrOpt('agent_driver',
               default='networking_vsphere.agent.ovsvapp_agent'
                       '.OVSvAppL2Agent',
               help='OVSvApp Agent implementation.'),
    cfg.StrOpt('network_manager',
               default='networking_vsphere.drivers.manager.'
                       'VcenterManager',
               help='Driver Manager implementation for '
                    'NetworkDriver.'),
    cfg.IntOpt('report_interval',
               default=4,
               help='Seconds between nodes reporting state to server.'),
    cfg.IntOpt('polling_interval',
               default=2,
               help='The number of seconds the agent will wait between '
                    'polling for local device changes.'),
    cfg.IntOpt('veth_mtu',
               default=1500,
               help='MTU size of veth interfaces.'),
    cfg.ListOpt('tunnel_types',
                default=[p_const.TYPE_VXLAN],
                help='Tunnel network types supported by the OVSvApp Agent.'),
    cfg.IntOpt('vxlan_udp_port',
               default=p_const.VXLAN_UDP_PORT,
               help='The UDP port to use for VXLAN tunnels.'),
    cfg.IntOpt('dont_fragment',
               default=True,
               help='Do not fragment.'),
    cfg.StrOpt('tunnel_bridge',
               default='br-tun',
               help='Tunnel Bridge for tunneling.'),
    cfg.StrOpt('local_ip',
               default='',
               help='Local IP address of VXLAN tunnel endpoint.'),
    cfg.BoolOpt('agent_maintenance',
                default=False,
                help='Turn on this flag during agent updates to help '
                     'prevent datapath outage.')
]

# OVSvApp Security Group related config read from ovsvapp_agent.ini.
SECURITYGROUP_OPTS = [
    cfg.StrOpt('security_bridge_mapping',
               default='br-sec',
               help='<security_bridge>:<phy_interface>'),
    cfg.BoolOpt('defer_apply',
                default=True,
                help='Enable defer_apply on security bridge.'),
    cfg.StrOpt('ovsvapp_firewall_driver',
               default='networking_vsphere.drivers.ovs_firewall.'
                       'OVSFirewallDriver',
               help='DriverManager implementation for '
                    'OVS based Firewall.')
]


def register_options():
    cfg.CONF.register_opts(VMWARE_OPTS, "VMWARE")
    cfg.CONF.register_opts(OVSVAPP_OPTS, "OVSVAPP")
    cfg.CONF.register_opts(SECURITYGROUP_OPTS, "SECURITYGROUP")


# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_vmware import vim_util


def build_recursive_traversal_spec(client_factory):
    # Recurse through all ResourcePools
    rp_to_rp = client_factory.create('ns0:TraversalSpec')
    rp_to_rp.name = 'rpToRp'
    rp_to_rp.type = 'ResourcePool'
    rp_to_rp.path = 'resourcePool'
    rp_to_rp.skip = False
    rp_to_vm = client_factory.create('ns0:TraversalSpec')
    rp_to_vm.name = 'rpToVm'
    rp_to_vm.type = 'ResourcePool'
    rp_to_vm.path = 'vm'
    rp_to_vm.skip = False
    spec_array_resource_pool = [client_factory.create('ns0:SelectionSpec'),
                                client_factory.create('ns0:SelectionSpec')]
    spec_array_resource_pool[0].name = 'rpToRp'
    spec_array_resource_pool[1].name = 'rpToVm'
    rp_to_rp.selectSet = spec_array_resource_pool

    # Traversal through resource pool branch
    cr_to_rp = client_factory.create('ns0:TraversalSpec')
    cr_to_rp.name = 'crToRp'
    cr_to_rp.type = 'ComputeResource'
    cr_to_rp.path = 'resourcePool'
    cr_to_rp.skip = False
    spec_array_compute_resource = [client_factory.create('ns0:SelectionSpec'),
                                   client_factory.create('ns0:SelectionSpec')]
    spec_array_compute_resource[0].name = 'rpToRp'
    spec_array_compute_resource[1].name = 'rpToVm'
    cr_to_rp.selectSet = spec_array_compute_resource

    # Traversal through host branch
    cr_to_h = client_factory.create('ns0:TraversalSpec')
    cr_to_h.name = 'crToH'
    cr_to_h.type = 'ComputeResource'
    cr_to_h.path = 'host'
    cr_to_h.skip = False

    # Traversal through hostFolder branch
    dc_to_hf = client_factory.create('ns0:TraversalSpec')
    dc_to_hf.name = 'dcToHf'
    dc_to_hf.type = 'Datacenter'
    dc_to_hf.path = 'hostFolder'
    dc_to_hf.skip = False
    spec_array_datacenter_host = [client_factory.create('ns0:SelectionSpec')]
    spec_array_datacenter_host[0].name = 'visitFolders'
    dc_to_hf.selectSet = spec_array_datacenter_host

    # Traversal through vmFolder branch
    dc_to_vmf = client_factory.create('ns0:TraversalSpec')
    dc_to_vmf.name = 'dcToVmf'
    dc_to_vmf.type = 'Datacenter'
    dc_to_vmf.path = 'vmFolder'
    dc_to_vmf.skip = False
    spec_array_datacenter_vm = [client_factory.create('ns0:SelectionSpec')]
    spec_array_datacenter_vm[0].name = 'visitFolders'
    dc_to_vmf.selectSet = spec_array_datacenter_vm

    # Traversal through datastore branch
    dc_to_ds = client_factory.create('ns0:TraversalSpec')
    dc_to_ds.name = 'dcToDs'
    dc_to_ds.type = 'Datacenter'
    dc_to_ds.path = 'datastore'
    dc_to_ds.skip = False
    spec_array_datacenter_ds = [client_factory.create('ns0:SelectionSpec')]
    spec_array_datacenter_ds[0].name = 'visitFolders'
    dc_to_ds.selectSet = spec_array_datacenter_ds

    # Recurse through all hosts
    h_to_vm = client_factory.create('ns0:TraversalSpec')
    h_to_vm.name = 'hToVm'
    h_to_vm.type = 'HostSystem'
    h_to_vm.path = 'vm'
    h_to_vm.skip = False
    spec_array_host_vm = [client_factory.create('ns0:SelectionSpec')]
    spec_array_host_vm[0].name = 'visitFolders'
    h_to_vm.selectSet = spec_array_host_vm

    # Recurse through all datastores
    ds_to_vm = client_factory.create('ns0:TraversalSpec')
    ds_to_vm.name = 'dsToVm'
    ds_to_vm.type = 'Datastore'
    ds_to_vm.path = 'vm'
    ds_to_vm.skip = False
    spec_array_datastore_vm = [client_factory.create('ns0:SelectionSpec')]
    spec_array_datastore_vm[0].name = 'visitFolders'
    ds_to_vm.selectSet = spec_array_datastore_vm

    # Recurse through the folders
    visit_folders = client_factory.create('ns0:TraversalSpec')
    visit_folders.name = 'visitFolders'
    visit_folders.type = 'Folder'
    visit_folders.path = 'childEntity'
    visit_folders.skip = False
    spec_array_visit_folders = [client_factory.create('ns0:SelectionSpec'),
                                client_factory.create('ns0:SelectionSpec'),
                                client_factory.create('ns0:SelectionSpec'),
                                client_factory.create('ns0:SelectionSpec'),
                                client_factory.create('ns0:SelectionSpec'),
                                client_factory.create('ns0:SelectionSpec'),
                                client_factory.create('ns0:SelectionSpec'),
                                client_factory.create('ns0:SelectionSpec'),
                                client_factory.create('ns0:SelectionSpec')]
    spec_array_visit_folders[0].name = 'visitFolders'
    spec_array_visit_folders[1].name = 'dcToHf'
    spec_array_visit_folders[2].name = 'dcToVmf'
    spec_array_visit_folders[3].name = 'crToH'
    spec_array_visit_folders[4].name = 'crToRp'
    spec_array_visit_folders[5].name = 'dcToDs'
    spec_array_visit_folders[6].name = 'hToVm'
    spec_array_visit_folders[7].name = 'dsToVm'
    spec_array_visit_folders[8].name = 'rpToVm'
    visit_folders.selectSet = spec_array_visit_folders

    # Add all of them here
    spec_array = [visit_folders, dc_to_vmf, dc_to_ds, dc_to_hf, cr_to_h,
                  cr_to_rp, rp_to_rp, h_to_vm, ds_to_vm, rp_to_vm]
    return spec_array


def get_object_properties(vim, collector, mobj, type, properties):
    """Gets the properties of the Managed object specified."""
    client_factory = vim.client.factory
    if mobj is None:
        return None
    usecoll = collector
    if usecoll is None:
        usecoll = vim.service_content.propertyCollector
    property_filter_spec = client_factory.create('ns0:PropertyFilterSpec')
    property_spec = client_factory.create('ns0:PropertySpec')
    property_spec.all = (properties is None or len(properties) == 0)
    property_spec.pathSet = properties
    property_spec.type = type
    object_spec = client_factory.create('ns0:ObjectSpec')
    object_spec.obj = mobj
    object_spec.skip = False
    property_filter_spec.propSet = [property_spec]
    property_filter_spec.objectSet = [object_spec]
    return retrieve_properties_ex(vim,
                                  usecoll,
                                  [property_filter_spec])


def get_dynamic_property(vim, mobj, type, property_name):
    """Gets a particular property of the Managed Object."""
    properties = get_dynamic_properties(vim, mobj, [property_name], type)
    property_value = None
    if property_name in properties:
        property_value = properties.get(property_name)
    return property_value


def get_dynamic_properties(vim, mobj, property_names, obj_type=None):
    """Gets specific properties of the Managed Object."""
    if not obj_type:
        obj_type = mobj._type
    obj_content = get_object_properties(
        vim, None, mobj, obj_type, property_names)
    properties = {}
    if obj_content:
        dynamic_properties = obj_content[0].propSet
        for dynamic_property in dynamic_properties:
            property_name = dynamic_property.name
            property_value = dynamic_property.val
            properties[property_name] = property_value
    return properties


def retrieve_properties_ex(vim, prop_coll, spec_set, max_count=500):
    """Retrieve properties.

    Retrieve properties using PropertyCollector.RetrievePropertiesEx
    and PropertyCollector.ContinueRetrievePropertiesEx
    args:
    :param vim: Vim object
    :param prop_coll: PropertyCollector MOR
    :param max_count: Max num of objects returned in a single call.
    """
    objcont = []
    client_factory = vim.client.factory
    opts = client_factory.create('ns0:RetrieveOptions')
    opts.maxObjects = max_count
    res = vim.RetrievePropertiesEx(prop_coll,
                                   specSet=spec_set,
                                   options=opts)
    while True:
        if res and res.objects:
            objcont.extend(res.objects)
        if hasattr(res, "token") and res.token:
            res = vim.ContinueRetrievePropertiesEx(prop_coll, token=res.token)
        else:
            break
    return objcont


def get_objects(vim, type, properties_to_collect=None, all=False):
    """Gets the list of objects of the type specified."""
    if not properties_to_collect:
        properties_to_collect = ["name"]

    client_factory = vim.client.factory
    trav_spec = vim_util.build_recursive_traversal_spec(client_factory)
    object_spec = vim_util.build_object_spec(client_factory,
                                             vim.service_content.rootFolder,
                                             [trav_spec])
    property_spec = vim_util.build_property_spec(
        client_factory, type_=type,
        properties_to_collect=properties_to_collect,
        all_properties=all)
    property_filter_spec = vim_util.build_property_filter_spec(client_factory,
                                                               [property_spec],
                                                               [object_spec])
    property_collector = vim.service_content.propertyCollector
    return retrieve_properties_ex(vim,
                                  property_collector,
                                  [property_filter_spec])


def get_prop_spec(client_factory, spec_type, properties):
    """Builds the Property Spec Object."""
    prop_spec = client_factory.create('ns0:PropertySpec')
    prop_spec.type = spec_type
    prop_spec.pathSet = properties
    return prop_spec


def get_obj_spec(client_factory, obj, select_set=None):
    """Builds the Object Spec object."""
    obj_spec = client_factory.create('ns0:ObjectSpec')
    obj_spec.obj = obj
    obj_spec.skip = False
    if select_set is not None:
        obj_spec.selectSet = select_set
    return obj_spec


def get_prop_filter_spec(client_factory, obj_spec, prop_spec):
    """Builds the Property Filter Spec Object."""
    prop_filter_spec = client_factory.create('ns0:PropertyFilterSpec')
    prop_filter_spec.propSet = prop_spec
    prop_filter_spec.objectSet = obj_spec
    return prop_filter_spec


def get_property_filter_specs(vim, property_dict, objects=None):
    client_factory = vim.client.factory
    object_specs = []
    if not objects:
        objects = [vim.service_content.rootFolder]
    for obj in objects:
        if obj.value == get_root_folder_id(vim):
            traversal_spec = [
                vim_util.build_recursive_traversal_spec(client_factory)]
        else:
            traversal_spec = build_recursive_traversal_spec(client_factory)
        object_spec = vim_util.build_object_spec(client_factory,
                                                 obj,
                                                 traversal_spec)
        object_specs.append(object_spec)

    property_specs = []
    for obj_type in property_dict:
        props = property_dict[obj_type]
        property_spec = vim_util.build_property_spec(
            client_factory, type_=obj_type, properties_to_collect=props)
        property_specs.append(property_spec)

    property_filter_spec = vim_util.build_property_filter_spec(client_factory,
                                                               property_specs,
                                                               object_specs)
    return property_filter_spec


def create_filter(vim, prop_filter_spec, collector=None):
    if not collector:
        collector = vim.service_content.propertyCollector
    return vim.CreateFilter(collector,
                            spec=prop_filter_spec,
                            partialUpdates=False)


def create_property_collector(vim, collector=None):
    if not collector:
        collector = vim.service_content.propertyCollector
    return vim.CreatePropertyCollector(collector)


def destroy_property_collector(vim, collector):
    if collector:
        return vim.DestroyPropertyCollector(collector)


def wait_for_updates_ex(vim, version, collector=None,
                        max_wait=85, max_update_count=-1):
    """Polling mechanism for property collection

        args:
        :param vim: Vim object
        :param version: version string
        :param collector: PropertyCollector MOR
        :param max_wait: Max time in seconds before the call returns
                       (Default set to 85 as 90 is the http socket timeout)
        :param max_update_count: Max num of ObjectUpdates returned
                               in a single call. Not set if <= 0
    """
    client_factory = vim.client.factory
    waitopts = client_factory.create('ns0:WaitOptions')
    waitopts.maxWaitSeconds = max_wait
    if max_update_count > 0:
        waitopts.maxObjectUpdates = max_update_count
    if not collector:
        collector = vim.service_content.propertyCollector
    return vim.WaitForUpdatesEx(collector,
                                version=version,
                                options=waitopts)


def cancel_wait_for_updates(vim, collector=None):
    if not collector:
        collector = vim.service_content.propertyCollector
    return vim.CancelWaitForUpdates(collector)


def get_properties_for_a_collection_of_objects(vim, type,
                                               obj_list, properties):
    """Gets the list of properties for the collection of objects."""
    client_factory = vim.client.factory
    if len(obj_list) == 0:
        return []
    prop_spec = get_prop_spec(client_factory, type, properties)
    lst_obj_specs = []
    for obj in obj_list:
        lst_obj_specs.append(get_obj_spec(client_factory, obj))
    prop_filter_spec = get_prop_filter_spec(client_factory,
                                            lst_obj_specs, [prop_spec])
    return retrieve_properties_ex(vim,
                                  vim.service_content.propertyCollector,
                                  [prop_filter_spec])


def get_search_index(vim):
    return vim.service_content.searchIndex


def find_by_inventory_path(vim, search_index, path):
    return vim.FindByInventoryPath(search_index, inventoryPath=path)


def get_root_folder_id(vim):
    return vim.service_content.rootFolder.value


def get_dv_switch_manager(vim):
    """Get reference of DistributedVirtualSwitchManager."""
    return vim.service_content.dvSwitchManager


def get_dvs_mor_by_uuid(vim, uuid):
    """Query DVS by UUID."""
    dvs_mgr = get_dv_switch_manager(vim)
    return vim.QueryDvsByUuid(dvs_mgr, uuid=uuid)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# Unit test for OVSvApp Mechanism Driver.

from neutron.extensions import portbindings
from neutron.plugins.ml2 import driver_api as api
from neutron.plugins.ml2.drivers.ovsvapp import mech_driver
from neutron.tests.unit.plugins.ml2 import _test_mech_agent as base

# TODO(romilg): Revisit to minimize dependency on ML2 tests.

from networking_vsphere.common import constants


class OVSvAppAgentMechanismBaseTestCase(base.AgentMechanismBaseTestCase):
    VIF_TYPE = portbindings.VIF_TYPE_OTHER
    CAP_PORT_FILTER = True
    AGENT_TYPE = constants.AGENT_TYPE_OVSVAPP

    GOOD_MAPPINGS = {'fake_physical_network': 'fake_bridge'}
    GOOD_TUNNEL_TYPES = ['vxlan']
    GOOD_CONFIGS = {'bridge_mappings': GOOD_MAPPINGS,
                    'tunnel_types': GOOD_TUNNEL_TYPES}

    BAD_MAPPINGS = {'wrong_physical_network': 'wrong_bridge'}
    BAD_TUNNEL_TYPES = ['bad_tunnel_type']
    BAD_CONFIGS = {'bridge_mappings': BAD_MAPPINGS,
                   'tunnel_types': BAD_TUNNEL_TYPES}

    AGENTS = [{'alive': True,
               'configurations': GOOD_CONFIGS,
               'host': 'host'}]
    AGENTS_DEAD = [{'alive': False,
                    'configurations': GOOD_CONFIGS,
                    'host': 'dead_host'}]
    AGENTS_BAD = [{'alive': False,
                   'configurations': GOOD_CONFIGS,
                   'host': 'bad_host_1'},
                  {'alive': True,
                   'configurations': BAD_CONFIGS,
                   'host': 'bad_host_2'}]

    def setUp(self):
        super(OVSvAppAgentMechanismBaseTestCase, self).setUp()
        self.driver = mech_driver.OVSvAppAgentMechanismDriver()
        self.driver.initialize()


class OVSvAppAgentMechanismGenericTestCase(
    OVSvAppAgentMechanismBaseTestCase,
    base.AgentMechanismGenericTestCase):
    pass


class OVSvAppAgentMechanismVlanTestCase(
    OVSvAppAgentMechanismBaseTestCase,
    base.AgentMechanismVlanTestCase):
    pass


class OVSvAppAgentMechanismVxlanTestCase(
    OVSvAppAgentMechanismBaseTestCase,
    base.AgentMechanismBaseTestCase):
    VXLAN_SEGMENTS = [{api.ID: 'unknown_segment_id',
                       api.NETWORK_TYPE: 'no_such_type'},
                      {api.ID: 'vxlan_segment_id',
                       api.NETWORK_TYPE: 'vxlan',
                       api.SEGMENTATION_ID: 1234}]

    def test_type_vxlan(self):
        context = base.FakePortContext(self.AGENT_TYPE,
                                       self.AGENTS,
                                       self.VXLAN_SEGMENTS)
        self.driver.bind_port(context)
        self._check_bound(context, self.VXLAN_SEGMENTS[1])

    def test_type_vxlan_bad(self):
        context = base.FakePortContext(self.AGENT_TYPE,
                                       self.AGENTS_BAD,
                                       self.VXLAN_SEGMENTS)
        self.driver.bind_port(context)
        self._check_unbound(context)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from networking_vsphere.common import constants
from networking_vsphere.common import model
from networking_vsphere.tests import base


class CommonModelTestCase(base.TestCase):

    def setUp(self):
        super(CommonModelTestCase, self).setUp()

    def test_model_entity(self):
        entity = model.ResourceEntity(key="key1",
                                      uuid="uuid1")
        self.assertEqual(entity.key, "key1", "Entity key does not match")
        self.assertEqual(entity.uuid, "uuid1", "Entity uuid does not match")

    def test_model_host(self):
        key = "host1"
        name = "hostname1"
        host = model.Host(name, key)
        self.assertFalse(host.uuid is None, "Host uuid is none")
        self.assertEqual(host.key, key, "Host key does not match")
        self.assertEqual(host.name, name, "Host name does not match")
        self.assertTrue(str(host) is not None)

    def test_model_physicalnic(self):
        key = "1-2-3"
        name = "eth0"
        mac_address = "ABC-DEFG-HIJK"
        config = None
        nic = model.PhysicalNic(name, mac_address, config, key)
        self.assertFalse(nic.uuid is None, "Phy nic uuid is none")
        self.assertEqual(nic.key, key, "Phy nic key does not match")
        self.assertEqual(nic.name, name, "Phy nic name does not match")
        self.assertEqual(nic.mac_address, mac_address,
                         "Phy nic mac_address does not match")
        self.assertEqual(nic.config, config,
                         "Phy nic config does not match")

    def test_model_virtualswitch(self):
        key = "1-2-3"
        name = "dvs1"
        pnic_name = "eth0"
        net_name = "net1"
        pnic = model.PhysicalNic(name="eth0", mac_address=None, config=None)
        network = model.Network(name=net_name, network_type=None)
        dvs = model.VirtualSwitch(name,
                                  pnics=[pnic],
                                  networks=[network],
                                  key=key)
        self.assertFalse(dvs.uuid is None, "DVS uuid is none")
        self.assertEqual(dvs.key, key, "DVS key does not match")
        self.assertEqual(dvs.name, name, "DVS name does not match")
        self.assertEqual(len(dvs.pnics), 1)
        self.assertEqual(dvs.pnics[0].name, pnic_name)
        self.assertEqual(len(dvs.networks), 1)
        self.assertEqual(dvs.networks[0].name, net_name)

    def test_model_network(self):
        key = "1-2-3"
        name = "net1"
        network_type = constants.NETWORK_VLAN
        vs_name = "dvs1"
        port_name = "port1"
        vlanIds = [1001]
        operation_mode = "mode1"
        vlan_type = "Native"
        vlan = model.Vlan(vlanIds, operation_mode, vlan_type)
        config = model.NetworkConfig(vlan)
        vs = model.VirtualSwitch(vs_name)
        port = model.Port(port_name, None, None, None, None)
        network = model.Network(name, network_type, config, [vs], [port], key)
        self.assertFalse(network.uuid is None, "Network uuid is none")
        self.assertEqual(network.key, key, "Network key does not match")
        self.assertEqual(network.name, name, "Network name does not match")
        self.assertEqual(network.network_type, network_type,
                         "Network network_type does not match")
        self.assertTrue(network.config is not None)
        self.assertTrue(network.config.vlan is not None)
        self.assertEqual(network.config.vlan.operation_mode, operation_mode)
        self.assertEqual(network.config.vlan.vlan_type, vlan_type)
        self.assertEqual(len(network.config.vlan.vlanIds), 1)
        self.assertEqual(network.config.vlan.vlanIds[0], vlanIds[0])
        self.assertEqual(len(network.vswitches), 1)
        self.assertEqual(network.vswitches[0].name, vs_name)
        self.assertEqual(len(network.ports), 1)
        self.assertEqual(network.ports[0].name, port_name)

    def test_model_port(self):
        key = "1-2-3"
        name = "net1"
        mac_address = "ABC-DEFG-HIJK"
        ipaddresses = ["100.100.10.1"]
        vswitch_uuid = "uuid1"
        network_uuid = "uuid2"
        vm_id = "uuid3"
        port_config = None
        port_status = None
        port = model.Port(name, mac_address, ipaddresses,
                          vswitch_uuid, vm_id, network_uuid, port_config,
                          port_status, key)
        self.assertFalse(port.uuid is None, "Port uuid is none")
        self.assertEqual(port.key, key, "Port key does not match")
        self.assertEqual(port.name, name, "Port name does not match")
        self.assertEqual(len(port.ipaddresses), 1)
        self.assertEqual(port.ipaddresses[0], ipaddresses[0])
        self.assertEqual(port.mac_address, mac_address,
                         "Port mac_address does not match")
        self.assertEqual(port.vswitch_uuid, vswitch_uuid,
                         "Port vswitch_uuid does not match")
        self.assertEqual(port.vm_id, vm_id,
                         "Port vm_id does not match")
        self.assertEqual(port.network_uuid, network_uuid,
                         "Port network_uuid does not match")

    def test_model_virtualnic(self):
        key = "1-2-3"
        mac_address = "ABC-DEFG-HIJK"
        port_uuid = "uuid1"
        vm_id = "uuid2"
        vm_name = "vm1"
        nic_type = "VMXNET"
        virtual_nic = model.VirtualNic(mac_address, port_uuid,
                                       vm_id, vm_name, nic_type, key)
        self.assertEqual(virtual_nic.key, key, "vnic key does not match")
        self.assertEqual(virtual_nic.mac_address, mac_address,
                         "vnic mac_address does not match")
        self.assertEqual(virtual_nic.port_uuid, port_uuid,
                         "vnic port_uuid does not match")
        self.assertEqual(virtual_nic.vm_id, vm_id,
                         "vnic vm_id does not match")
        self.assertEqual(virtual_nic.vm_name, vm_name,
                         "vnic vm_name does not match")
        self.assertEqual(virtual_nic.nic_type, nic_type,
                         "vnic nic_type does not match")

    def test_model_virtualmachine(self):
        key = "1-2-3"
        vm_name = "vm1"
        vm_uuid = "uuid1"
        vnic = model.VirtualNic(None, None, vm_uuid, vm_name, None, None)
        vm = model.VirtualMachine(vm_name, [vnic], vm_uuid, key)
        self.assertEqual(vm.uuid, vm_uuid, "vm uuid does not match")
        self.assertEqual(vm.key, key, "vm key does not match")
        self.assertEqual(vm.name, vm_name, "vm name does not match")
        self.assertEqual(len(vm.vnics), 1, "vm vnics size does not match")
        self.assertEqual(vm.vnics[0].vm_id, vm_uuid,
                         "vm vnics vm_id does not match")

    def test_model_virtualmachine_none_uuid(self):
        key = "1-2-3"
        vm_name = "vm1"
        vm = model.VirtualMachine(vm_name, None, None, key)
        self.assertTrue(vm.uuid is not None, "vm uuid is none")
        self.assertEqual(vm.key, key, "vm key does not match")
        self.assertEqual(vm.name, vm_name, "vm name does not match")

    def test_model_event(self):
        event_type = 'VM_CREATED'
        src_obj = "vm1"
        hostname = 'host1'
        clustername = 'cluster1'
        clusterid = 'domain-xyz'
        host_changed = False
        event = model.Event(event_type, src_obj, None,
                            hostname, clustername, clusterid,
                            host_changed)
        self.assertEqual(event.event_type, event_type,
                         "event event_type does not match")
        self.assertEqual(event.host_name, hostname,
                         "event host_name does not match")
        self.assertEqual(event.cluster_name, clustername,
                         "event cluster_name does not match")
        self.assertEqual(event.cluster_id, clusterid,
                         "event cluster_id does not match")
        self.assertEqual(event.host_changed, host_changed,
                         "event host_changed does not match")



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import contextlib

import eventlet
import mock

from networking_vsphere.drivers import dvs_driver
from networking_vsphere.drivers import manager
from networking_vsphere.drivers import vc_driver
from networking_vsphere.tests import base
from networking_vsphere.tests.unit.drivers import fake_driver
from networking_vsphere.tests.unit.utils import stubs
from networking_vsphere.utils import vim_session


class TestVcenterManager(base.TestCase):

    def setUp(self):
        super(TestVcenterManager, self).setUp()
        self.callback = fake_driver.MockCallback()
        self.manager = manager.VcenterManager(self.callback)
        self.useFixture(stubs.FakeVmware())
        self.LOG = manager.LOG

    def test_parse_mapping(self):
        cluster_dvs_list = self.manager._parse_mapping("abc:123")
        self.assertEqual(len(cluster_dvs_list), 1)

    def test_parse_mapping_multiple(self):
        cluster_dvs_list = self.manager._parse_mapping("abc:123,def:456")
        self.assertEqual(len(cluster_dvs_list), 2)

    def test_initialize_driver_noconf(self):
        with contextlib.nested(
            mock.patch.object(vim_session.ConnectionHandler, "stop",
                              return_value=None),
            mock.patch.object(self.LOG, "error")
        ) as (stop_conn, log_error):
            self.manager.initialize_driver()
            self.assertIsNone(self.manager.driver)
            self.assertTrue(log_error.called)

    def test_initialize_driver(self):
        fake_tuple = ["dc/host/cluster1:dvs1"]
        self.flags(vcenter_ip="vcenter.test.com", group='VMWARE')
        self.flags(vcenter_username="fake_user", group='VMWARE')
        self.flags(vcenter_password="fake_pass", group='VMWARE')
        self.flags(vcenter_api_retry_count="1", group='VMWARE')
        self.flags(wsdl_location="http://fake.test.com", group='VMWARE')
        self.flags(cluster_dvs_mapping=fake_tuple, group='VMWARE')
        with contextlib.nested(
            mock.patch.object(vim_session.ConnectionHandler, "stop",
                              return_value=None),
            mock.patch.object(eventlet, "spawn"),
            mock.patch.object(vc_driver.VCNetworkDriver, "add_cluster",
                              return_value=True)):
                self.assertEqual(len(self.manager.cluster_switch_mapping), 0)
                self.manager.initialize_driver()
                self.assertIsNotNone(self.manager.driver)
                self.assertEqual(len(self.manager.cluster_switch_mapping), 1)

    def test_start(self):
        self.manager.driver = None
        self.assertIsNone(self.manager.start())

    def test_pause(self):
        with mock.patch.object(dvs_driver.DvsNetworkDriver, "pause") as pause:
            self.manager.driver = dvs_driver.DvsNetworkDriver()
            self.manager.pause()
            self.assertTrue(pause.called)

    def test_pause_none_driver(self):
        self.manager.driver = None
        self.assertIsNone(self.manager.pause())

    def test_stop(self):
        with contextlib.nested(
            mock.patch.object(vim_session.ConnectionHandler, "stop",
                              return_value=None),
            mock.patch.object(dvs_driver.DvsNetworkDriver, "stop")
        ) as (conn_stop, dvs_stop):
                self.manager.driver = dvs_driver.DvsNetworkDriver()
                self.manager.stop()
                self.assertTrue(dvs_stop.called)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from networking_vsphere.drivers import base_manager
from networking_vsphere.tests import base


class DriverManagerTestCase(base.TestCase):

    def setUp(self):
        super(DriverManagerTestCase, self).setUp()
        self.base_manager = base_manager.DriverManager()

    def test_initialize_driver(self):
        self.assertRaises(NotImplementedError,
                          self.base_manager.initialize_driver)

    def test_get_driver(self):
        self.assertIsNone(self.base_manager.get_driver())

    def test_start(self):
        self.assertIsNone(self.base_manager.start())

    def test_pause(self):
        self.assertIsNone(self.base_manager.pause())

    def test_stop(self):
        self.assertIsNone(self.base_manager.stop())



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time
import uuid

_CLASSES = ['session', 'VirtualMachine', 'HostSystem', 'HostNetworkSystem',
            'ClusterComputeResource', 'Network', 'DistributedVirtualSwitch',
            'DistributedVirtualPortgroup', 'Task']

_db_content = {}


class Constants:
    VM_UUID = "1111-2222-3333-4444"
    VM_NAME = "TEST_VIRTUAL_MACHINE"
    VM_MAC = "11:22:33:44:55:ef"
    HOST_NAME = "TEST_HOST"
    PORTGROUP_NAME = "6d382cca-d8c6-42df-897d-9b6a99d4c04d"


def reset():
    """Resets the db contents."""
    for c in _CLASSES:
        _db_content[c] = {}
    create_network()
    create_host_network_system()
    create_host()
    create_virtual_machine()
    create_cluster_compute_resource()
    create_distributed_virtual_portgroup()
    create_distributed_virtual_switch()


def cleanup():
    """Clear the db contents."""
    for c in _CLASSES:
        _db_content[c] = {}


def _create_object(table, table_obj):
    """Create an object in the db."""
    _db_content[table][table_obj.value] = table_obj


class Prop(object):
    """Property Object base class."""

    def __init__(self):
        self.name = None
        self.val = None


class ManagedObject(object):
    """Managed Data Object base class."""

    def __init__(self, name="ManagedObject", obj_ref=None):
        """Sets the obj property which acts as a reference to the object."""
        super(ManagedObject, self).__setattr__('objName', name)
        if obj_ref is None:
            obj_ref = str(uuid.uuid4())
        object.__setattr__(self, 'obj', self)
        object.__setattr__(self, 'propSet', [])
        object.__setattr__(self, 'value', obj_ref)
        object.__setattr__(self, '_type', name)

    def set(self, attr, val):
        """Sets an attribute value.

        Not using the __setattr__ directly for we want to set attributes
        of the type 'a.b.c' and using this function class we set the same.
        """
        self.__setattr__(attr, val)

    def get(self, attr):
        """Gets an attribute.

        Used as an intermediary to get nested property like 'a.b.c' value.
        """
        return self.__getattr__(attr)

    def __setattr__(self, attr, val):
        for prop in self.propSet:
            if prop.name == attr:
                prop.val = val
                return
        elem = Prop()
        elem.name = attr
        elem.val = val
        self.propSet.append(elem)

    def __getattr__(self, attr):
        for elem in self.propSet:
            if elem.name == attr:
                return elem.val
        msg = _("Property %(attr)s not set for the managed object %(name)s")
        raise AttributeError(msg % {'attr': attr, 'name': self.objName})


class DataObject(object):
    """Data object base class."""
    pass


class Network(ManagedObject):

    """Network class."""

    def __init__(self):
        super(Network, self).__init__("Network")
        self.set("summary.name", "vmnet0")


class ClusterComputeResource(ManagedObject):
    """ClusterComputeResource class."""

    def __init__(self, **kwargs):
        super(ClusterComputeResource, self).__init__("ClusterComputeResource")
        host = _db_content["HostSystem"].values()[0]
        host.set("parent", self)
        host_sytem = DataObject()
        host_sytem.ManagedObjectReference = [host]
        self.set("host", host_sytem)


class HostNetworkSystem(ManagedObject):
    """HostNetworkSystem class."""

    def __init__(self):
        super(HostNetworkSystem, self).__init__("HostNetworkSystem")
        self.set("name", "networkSystem")


class HostSystem(ManagedObject):
    """Host System class."""

    def __init__(self):
        super(HostSystem, self).__init__("HostSystem")
        self.set("name", Constants.HOST_NAME)

        if _db_content.get("HostNetworkSystem", None) is None:
            create_host_network_system()
        host_net_key = _db_content["HostNetworkSystem"].keys()[0]
        host_net_sys = _db_content["HostNetworkSystem"][host_net_key].value
        self.set("configManager.networkSystem", host_net_sys)

        if _db_content.get("Network", None) is None:
            create_network()
        net_ref = (_db_content["Network"]
                   [_db_content["Network"].keys()[0]].obj)
        network_do = DataObject()
        network_do.ManagedObjectReference = [net_ref]
        self.set("network", network_do)


class VirtualDisk(DataObject):
    """Virtual Disk class."""

    def __init__(self):
        super(VirtualDisk, self).__init__()
        self.key = 0
        self.unitNumber = 0


class VirtualPCNet32(DataObject):
    """VirtualPCNet32 class."""

    def __init__(self):
        super(VirtualPCNet32, self).__init__()
        self.key = 4000


class VirtualMachine(ManagedObject):
    """Virtual Machine class."""

    def __init__(self, **kwargs):
        super(VirtualMachine, self).__init__("VirtualMachine")
        self.set("name", Constants.VM_NAME)
        config = DataObject()
        extra_config = DataObject()
        extra_config_option = DataObject()
        extra_config_option.key = "nvp.vm-uuid"
        extra_config_option.value = Constants.VM_UUID
        extra_config.OptionValue = [extra_config_option]
        config.extraConfig = extra_config
        self.set("config", config)
        self.set('config.extraConfig', extra_config)
        self.set('config.extraConfig["nvp.vm-uuid"]', extra_config_option)
        runtime = DataObject()
        host_ref = _db_content["HostSystem"][
            _db_content["HostSystem"].keys()[0]]
        runtime.host = host_ref
        self.set("runtime", runtime)
        self.set("runtime.host", runtime.host)
        nic = VirtualPCNet32()
        nic.macAddress = "00:99:88:77:66:ab"
        devices = DataObject()
        devices.VirtualDevice = [nic, VirtualDisk()]
        self.set("config.hardware.device", devices)


class DistributedVirtualPortgroup(ManagedObject):
    """DistributedVirtualPortgroup class."""

    def __init__(self):
        super(DistributedVirtualPortgroup, self).__init__(
            "DistributedVirtualPortgroup")
        self.set("summary.name", Constants.PORTGROUP_NAME)

        vm_ref = _db_content["VirtualMachine"].values()[0]
        vm_object = DataObject()
        vm_object.ManagedObjectReference = [vm_ref]
        self.set("vm", vm_object)
        self.set("tag", None)
        config = DataObject()
        config.key = self.value
        config.name = Constants.PORTGROUP_NAME
        defaultPortConfig = DataObject()
        vlan = DataObject()
        vlan.vlanId = 100
        defaultPortConfig.vlan = vlan
        config.defaultPortConfig = defaultPortConfig
        self.set("config", config)
        self.set("portKeys", ["18001",
                              "18002",
                              "18003",
                              "18004"])


class DistributedVirtualSwitch(ManagedObject):
    """DistributedVirtualSwitch class."""

    def __init__(self):
        super(DistributedVirtualSwitch, self).__init__(
            "DistributedVirtualSwitch")
        self.set("name", "test_dvs")
        host_ref = _db_content["HostSystem"].values()[0]
        dvs_host_member_config_info = DataObject()
        dvs_host_member_config_info.host = host_ref
        dvs_host_member = DataObject()
        dvs_host_member.config = dvs_host_member_config_info
        self.set("config.host", [[dvs_host_member]])
        self.set("uuid", "fake_dvs")
        pg = _db_content["DistributedVirtualPortgroup"].values()[0]
        pg_config = pg.config
        pg_config.distributedVirtualSwitch = self
        pg_object = DataObject()
        pg_object.ManagedObjectReference = [pg]
        self.set("portgroup", pg_object)
        nic = VirtualPCNet32()
        backing = DataObject()
        backing.port = DataObject()
        backing.port.portgroupKey = pg.value
        backing.port.portKey = pg.portKeys[0]
        backing.port.switchUuid = self.uuid
        nic.macAddress = Constants.VM_MAC
        nic.backing = backing
        vm = pg = _db_content["VirtualMachine"].values()[0]
        vm.get("config.hardware.device").VirtualDevice.append(nic)


class Task(ManagedObject):
    """Task class."""

    def __init__(self, task_name, state="running"):
        super(Task, self).__init__("Task")
        info = DataObject
        info.name = task_name
        info.state = state
        info.key = self.value
        if state == "error":
            error_do = DataObject
            error_do.localizedMessage = "fake_error"
            info.error = error_do
        self.set("info", info)


def create_network():
    network = Network()
    _create_object('Network', network)
    return network


def create_host_network_system():
    host_net_system = HostNetworkSystem()
    _create_object("HostNetworkSystem", host_net_system)
    return host_net_system


def create_host():
    host_system = HostSystem()
    _create_object('HostSystem', host_system)
    return host_system


def create_cluster_compute_resource():
    cluster = ClusterComputeResource()
    _create_object('ClusterComputeResource', cluster)
    return cluster


def create_virtual_machine():
    virtual_machine = VirtualMachine()
    _create_object('VirtualMachine', virtual_machine)
    return virtual_machine


def create_distributed_virtual_portgroup():
    pg = DistributedVirtualPortgroup()
    _create_object("DistributedVirtualPortgroup", pg)
    return pg


def create_distributed_virtual_switch():
    dvs = DistributedVirtualSwitch()
    _create_object("DistributedVirtualSwitch", dvs)
    return dvs


def create_task(task_name, state="running"):
    task = Task(task_name, state)
    _create_object("Task", task)
    return task


def is_task_done(task_name):
    for task in _db_content["Task"].values():
        if task.info.name == task_name:
            return True
    return False


class FakeFactory(object):
    """Fake factory class with create method."""

    def create(self, obj_name):
        """Creates a namespace object."""
        return DataObject()


class FakeVim(object):
    """Fake VIM Class."""

    def __init__(self, protocol="https", host="localhost", trace=None):
        """Initializes a fake Vim object.

        Sets the service contents and cookies for the session.
        """
        self._session = None
        self.client = DataObject()
        self.client.factory = FakeFactory()

        service_content = self.client.factory.create('ns0:ServiceContent')
        service_content.propertyCollector = "PropCollector"
        service_content.sessionManager = "SessionManager"
        service_content.rootFolder = DataObject()
        service_content.rootFolder.value = "RootFolder"
        service_content.rootFolder._type = "Folder"
        service_content.dvSwitchManager = "DistributedVirtualSwitchManager"
        service_content.searchIndex = "SearchIndex"
        self.service_content = service_content

    def get_service_content(self):
        return self.service_content

    def _login(self):
        """Logs in and sets the session object in the db."""
        self._session = str(uuid.uuid4())
        session = DataObject()
        session.key = self._session
        _db_content['session'][self._session] = session
        return session

    def _retrieve_properties(self, method, *args, **kwargs):
        """Retrieves properties based on the type."""
        spec_set = kwargs.get("specSet")[0]
        type = spec_set.propSet[0].type
        properties = spec_set.propSet[0].pathSet
        objectSets = spec_set.objectSet
        lst_ret_objs = []
        for objectSet in objectSets:
            try:
                obj_ref = objectSet.obj
                if obj_ref == self.get_service_content().rootFolder:
                    for mdo_ref in _db_content[type]:
                        mdo = _db_content[type][mdo_ref]
                        temp_mdo = ManagedObject(mdo.objName, mdo.value)
                        for prop in properties:
                            temp_mdo.set(prop, mdo.get(prop))
                        lst_ret_objs.append(temp_mdo)
                else:
                    if isinstance(obj_ref, ManagedObject):
                        obj_ref = obj_ref.value
                    if obj_ref in _db_content[type]:
                        mdo = _db_content[type][obj_ref]
                        temp_mdo = ManagedObject(mdo.objName, obj_ref)
                        for prop in properties:
                            temp_mdo.set(prop, mdo.get(prop))
                        lst_ret_objs.append(temp_mdo)
            except Exception:
                continue
        if method == "RetrievePropertiesEx":
            res = DataObject()
            res.objects = lst_ret_objs
            return res
        else:
            return lst_ret_objs

    def _delete_port_group(self, method, *args, **kwargs):
        """Deletes a portgroup."""
        pg_key = _db_content["DistributedVirtualPortgroup"].keys()[0]
        del _db_content["DistributedVirtualPortgroup"][pg_key]
        task_mdo = create_task(method, "success")
        return task_mdo.obj

    def _just_return_task(self, method):
        """Fakes a task return."""
        task_mdo = create_task(method, "success")
        return task_mdo.obj

    def _just_return(self):
        """Fakes a return."""
        return

    def _query_dvs_by_uuid(self, method, *args, **kwargs):
        """Query DVS by uuid."""
        uuid = kwargs.get("uuid")
        for dvs in _db_content["DistributedVirtualSwitch"].values():
            if dvs.uuid == uuid:
                return dvs
        return None

    def _find_by_inventory_path(self, method, *args, **kwargs):
        path = kwargs.get("inventoryPath")
        try:
            return _db_content[path].values()[0]
        except KeyError:
            return None

    def _reconfigure_dv_port_task(self, method, *args, **kwargs):
        vds_ref = _db_content["DistributedVirtualSwitch"].values()[0]
        specs = kwargs.get("port")
        for spec in specs:
            found = False
            portKey = spec.key
            for pg in vds_ref.portgroup.ManagedObjectReference:
                if portKey in pg.portKeys:
                    found = True
                    break
            if not found:
                raise Exception("The object or item referred"
                                " to could not be found.")
        task_mdo = create_task(method, "success")
        return task_mdo.obj

    def _wait_for_updates(self, method, *args, **kwargs):
        version = kwargs.get("version")
        if not version:
            updateSet = DataObject()
            updateSet.version = 1
            filterSet = []
            updateSet.filterSet = filterSet
            propFilterUpdate = DataObject()
            filterSet.append(propFilterUpdate)
            objectSet = []
            propFilterUpdate.objectSet = objectSet
            for vm in _db_content["VirtualMachine"].values():
                objectUpdate = DataObject()
                objectUpdate.obj = vm
                objectUpdate.kind = "enter"
                changeSet = []
                objectUpdate.changeSet = changeSet
                for prop in vm.propSet:
                    changeSet.append(prop)
                objectSet.append(objectUpdate)
            return updateSet
        else:
            time.sleep(0)
            return None

    def __getattr__(self, attr_name):
        if attr_name == "Login":
            return lambda *args, **kwargs: self._login()
        elif attr_name == "RetrievePropertiesEx":
            return (lambda *args, **kwargs:
                    self._retrieve_properties(attr_name, *args, **kwargs))
        elif attr_name == "Destroy_Task":
            return (lambda *args, **kwargs:
                    self._delete_port_group(attr_name, *args, **kwargs))
        elif attr_name == "AddDVPortgroup_Task":
            return lambda *args, **kwargs: self._just_return_task(attr_name)
        elif attr_name == "ReconfigureDVPort_Task":
            return (lambda *args, **kwargs:
                    self._reconfigure_dv_port_task(attr_name, *args, **kwargs))
        elif attr_name == "QueryDvsByUuid":
            return (lambda *args, **kwargs:
                    self._query_dvs_by_uuid(attr_name, *args, **kwargs))
        elif attr_name == "CreateFilter":
            return lambda *args, **kwargs: "Filter"
        elif attr_name == "DestroyPropertyFilter":
            return lambda *args, **kwargs: self._just_return()
        elif attr_name == "FindByInventoryPath":
            return (lambda *args, **kwargs:
                    self._find_by_inventory_path(attr_name, *args, **kwargs))
        elif attr_name == "WaitForUpdatesEx":
            return (lambda *args, **kwargs:
                    self._wait_for_updates(attr_name, *args, **kwargs))
        elif attr_name == "CreatePropertyCollector":
            return lambda *args, **kwargs: "PropertyCollector"
        elif attr_name == "DestroyPropertyCollector":
            return lambda *args, **kwargs: self._just_return()


class FakeDynamicPropertyObject(object):
    """Fake Dynamic Property object."""

    def __init__(self):
        self.name = "name"
        self.val = "val"


class FakeObjectContent(object):
    """Fake ObjectContent object."""

    def __init__(self):
        self.propSet = [FakeDynamicPropertyObject()]


class FakeRetrieveResultObject(object):
    """Fake RetrieveResult object."""

    def __init__(self):
        self.objects = [FakeObjectContent()]
        self.token = "1234"


class FakeVimUtil(object):
    """Fake Vim util object class."""

    def __init__(self):
        self.client = DataObject()
        self.client.factory = FakeFactory()

        service_content = self.client.factory.create('ns0:ServiceContent')
        service_content.propertyCollector = "PropCollector"
        service_content.rootFolder = DataObject()
        service_content.rootFolder.value = "RootFolder"
        self.service_content = service_content

        self.RetrievePropertiesExCalled = False
        self.ContinueRetrievePropertiesExCalled = False
        self.CreateFilterCalled = False
        self.CreatePropertyCollectorCalled = False

    def RetrievePropertiesEx(self, prop_coll, specSet, options):
        self.RetrievePropertiesExCalled = True
        res = FakeRetrieveResultObject()
        return res

    def ContinueRetrievePropertiesEx(self, prop_coll, token=None):
        self.ContinueRetrievePropertiesExCalled = True
        return



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sqlalchemy as sa

from neutron.db import model_base


class ClusterVNIAllocations(model_base.BASEV2):
    """Represents a VXLAN Network to Local VLAN binding in a cluster."""

    __tablename__ = "ovsvapp_cluster_vni_allocations"

    vcenter_id = sa.Column(sa.String(36), nullable=False, primary_key=True)
    cluster_id = sa.Column(sa.String(255), nullable=False, primary_key=True)
    lvid = sa.Column(sa.Integer, nullable=False, autoincrement=False,
                     primary_key=True)
    network_id = sa.Column(sa.String(36), default=None)
    allocated = sa.Column(sa.Boolean, nullable=False, default=False)
    network_port_count = sa.Column(sa.Integer, nullable=False, default=0)

    def __repr__(self):
        """Cluster VNI allocations representation."""
        return ("<ClusterVNIAllocation(%s,%s,%s,%s,%s,%s)>." %
                (self.vcenter_id, self.cluster_id, self.lvid, self.network_id,
                 self.allocated, self.network_port_count))

    def __eq__(self, other):
        """Compare only the allocation."""
        return (self.vcenter_id == other.vcenter_id and
                self.cluster_id == other.cluster_id and
                self.lvid == other.lvid and
                self.network_id == other.network_id and
                self.allocated == other.allocated and
                self.network_port_count == other.network_port_count)
